﻿Imports BrightIdeasSoftware
Imports SFDL.Core
Imports SFDL.File
Imports System.Xml
Imports SFDL.Loader.OLV_ItemEnums

Module Helper

    Const MAXSFDLFILEVERSION As Integer = 6
    Const LEGACYSFDLFILEVERSION As Integer = 5

    Public Event AddTabPage(ByVal _tab_name As String, ByVal _tab_description As String)
    Public Event OpenSFDLFileComplete(ByVal _olv_file_item_list As List(Of OLV_Item), ByVal _sfdl_session_uid As String) 'Event für das einzelne File
    Public Event OpenSFDLComplete(ByVal _taskid As Integer, ByVal _bulk_package As Boolean) 'Event für den kompletten Vorgang
    Public Event OpenSFDLSessionComplete(ByVal _olv_file_item_list As List(Of OLV_Item), ByVal _sfdl_session As SFDLSession)

#Region "SFDL Datei öffnen"

    Friend Sub OpenSFDLFiles(ByVal _sfdl_file_path_list() As String, ByVal _taskid As Integer)

        Dim _tools As New Tools

        Dim _sfdl_file As New SFDLFile
        Dim _legacy_sfdl_file As New LegacyFile.SFDL_File
        Dim _sfdl_file_version As Integer
        Dim _decrypt_password As String = String.Empty
        Dim _legacy_mode As Boolean = False

        Dim _sfdl_file_uid As String = String.Empty
        Dim _sfdl_session As New SFDLSession
        Dim _olv_file_item_list As New List(Of OLV_Item)

        Dim _bulk_package_found As Boolean = False
        Dim _bulk_package_list As New List(Of SFDLPackage)

        Dim _logger As New Logger

        _logger = frm_main.GetLoggerInstance

        For Each _sfdl_file_path In _sfdl_file_path_list

            Try

                'Prüfen ob die SFDL Container Version kompatibel ist

                _sfdl_file_version = GetSFDLFileVersion(_sfdl_file_path)

                If _sfdl_file_version > MAXSFDLFILEVERSION Then
                    Throw New Exception("Die SFDL Loader ist nicht kompatibel zu dieser SFDL Datei. Bitte lade dir die aktuellste Version herunter!")
                End If

                'Legacy SFDL verarbeiten

                If _sfdl_file_version <= LEGACYSFDLFILEVERSION Then

                    _logger.WriteLog("OpenSFDLFiles", "Legacy SFDL Ermittelt")

                    _legacy_mode = True

                    _legacy_sfdl_file = CType(_tools.XMLDeSerialize(_legacy_sfdl_file, _sfdl_file_path), LegacyFile.SFDL_File)

                    If String.IsNullOrWhiteSpace(_legacy_sfdl_file.GetSetDescription) Then
                        _legacy_sfdl_file.GetSetDescription = IO.Path.GetFileNameWithoutExtension(_sfdl_file_path)
                    End If

                    If _legacy_sfdl_file.GetSetEncryptet = True Then

                        frm_opensfdl_encrypted.TopMost = True
                        frm_opensfdl_encrypted.SetTestDecryptString = _legacy_sfdl_file.GetSetConnectionInfo.GetSetHost

                        If frm_opensfdl_encrypted.ShowDialog() = DialogResult.OK Then
                            _decrypt_password = frm_opensfdl_encrypted.GetPassword
                            frm_opensfdl_encrypted.Dispose()
                        Else
                            Throw New UserCancelException
                        End If

                        _legacy_sfdl_file = DecryptLegacyFile(_legacy_sfdl_file, _decrypt_password)

                    End If

                Else 'Normale Aktuelle SFDL Datei verarbeiten

                    _sfdl_file = CType(_tools.XMLDeSerialize(_sfdl_file, _sfdl_file_path), SFDLFile)

                    If String.IsNullOrWhiteSpace(_sfdl_file.Description) Then
                        _sfdl_file.Description = IO.Path.GetFileNameWithoutExtension(_sfdl_file_path)
                    End If

                    If _sfdl_file.Encrypted = True Then

                        frm_opensfdl_encrypted.TopMost = True
                        frm_opensfdl_encrypted.SetTestDecryptString = _sfdl_file.ConnectionInfo.Host

                        If Not frm_opensfdl_encrypted.ShowDialog() >= DialogResult.Cancel Then
                            _decrypt_password = frm_opensfdl_encrypted.GetPassword
                            frm_opensfdl_encrypted.Dispose()
                        End If

                    End If

                End If

                'Legacy SFDL File in neues Format konvertieren
                If _legacy_mode = True Then
                    _sfdl_file = ConvertLegacyFile(_legacy_sfdl_file)
                End If

                _sfdl_file_uid = _tools.GenerateSFDLFileUID(_sfdl_file_path)

                If frm_main.GetSFDLSessionList.ContainsKey(_sfdl_file_uid) = True Then
                    Throw New Exception("Die SFDL Datei ist bereits geladen!")
                End If

                With _sfdl_session

                    .SessionState = SFDLSessionEnums.SFDLSessionState.Queued
                    .SFDLFile = _sfdl_file
                    .SFDLSessionName = _sfdl_file_uid
                    .SFDLFileName = IO.Path.GetFileNameWithoutExtension(_sfdl_file_path)

                End With

                frm_main.AddSFDLSession(New KeyValuePair(Of String, SFDLSession)(_sfdl_file_uid, _sfdl_session))

                RaiseEvent AddTabPage(_sfdl_file_uid, _sfdl_file.Description)

                For Each _package In _sfdl_file.Packages

                    If _package.BulkFolderMode = False Then

                        For Each _file In _package.FileList

                            Dim _olv_file_item As New OLV_Item

                            With _olv_file_item

                                .DirectoryPath = _file.DirectoryPath
                                .DirectoryRoot = _file.DirectoryRoot
                                .FileFullPath = _file.FileFullPath
                                .FileName = _file.FileName
                                .FileSize = _file.FileSize
                                .FileHashType = _file.FileHashType
                                .FileHash = _file.FileHash
                                .PackageName = _file.PackageName
                                .Progress = 0
                                .Status = Status.Queued
                                .SFDLSessionName = _sfdl_file_uid

                            End With

                            _olv_file_item_list.Add(_olv_file_item)

                        Next

                    Else
                        _bulk_package_found = True
                        _bulk_package_list.Add(_package)
                    End If

                Next

                RaiseEvent OpenSFDLFileComplete(_olv_file_item_list, _sfdl_session.SFDLSessionName)


                For Each _bulk_package In _bulk_package_list

                    Dim _thread_handler As New ThreadHandler
                    Dim _thread As Threading.Thread

                    _thread = New Threading.Thread(Sub() _thread_handler.GetBulkFileList(_bulk_package, _sfdl_file.ConnectionInfo, _sfdl_session.SFDLSessionName, _taskid))

                    _thread.Start()

                Next


            Catch ex As Exception
                _logger.WriteLog("OpenSFDLFiles", ex.Message, LogType.Fehler)
                MessageBox.Show("Fehler beim Öffnen der SFDL Dateie(n)" & vbNewLine & "Fehlermeldung: " & ex.Message, "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try

        Next

        RaiseEvent OpenSFDLComplete(_taskid, _bulk_package_found)

    End Sub

    Private Function GetSFDLFileVersion(ByVal _filename As String) As Integer

        Dim _xml As New XmlDocument
        Dim _version As Integer
        Dim _logger As New Logger

        _logger = frm_main.GetLoggerInstance

        _xml.Load(_filename)

        Try
            'Zuerst prüfen ob es sich um eine alte SFDL Version handelt

            For Each _element As XmlNode In _xml.GetElementsByTagName("GetSetFileVersion")
                _version = Integer.Parse(_element.InnerText.ToString)
            Next

            For Each _element As XmlNode In _xml.GetElementsByTagName("SFDLFileVersion")
                _version = Integer.Parse(_element.InnerText.ToString)
            Next

        Catch ex As Exception
            _version = 0
        End Try

        _logger.WriteLog("GetSFDLFileVersion", "SFDL File Version: " & _version)

        Return _version

    End Function

    Private Function DecryptLegacyFile(ByVal _legacy_sfdl_file As LegacyFile.SFDL_File, ByVal _password As String) As LegacyFile.SFDL_File

        Dim _rt_sfdl_file As New LegacyFile.SFDL_File
        Dim _rt_file_list As New List(Of LegacyFile.FileInfo)
        Dim _crypto As New SFDL.Crypto.AES

        _rt_sfdl_file.GetSetDescription = _crypto.DecryptString(_legacy_sfdl_file.GetSetDescription, _password)
        _rt_sfdl_file.GetsetUploader = _crypto.DecryptString(_legacy_sfdl_file.GetsetUploader, _password)

        _rt_sfdl_file.GetSetFileVersion = _legacy_sfdl_file.GetSetFileVersion
        _rt_sfdl_file.GetSetMaxDownloadThread = _legacy_sfdl_file.GetSetMaxDownloadThread

        For Each _file_info In _legacy_sfdl_file.GetSetFileInfoList

            Dim _rt_legacy_file_info As New LegacyFile.FileInfo

            _rt_legacy_file_info.GetSetDirectoryPath = _crypto.DecryptString(_file_info.GetSetDirectoryPath, _password)
            _rt_legacy_file_info.GetSetDirectoryRoot = _crypto.DecryptString(_file_info.GetSetDirectoryRoot, _password)
            _rt_legacy_file_info.GetSetFileFullPath = _crypto.DecryptString(_file_info.GetSetFileFullPath, _password)
            _rt_legacy_file_info.GetSetFileName = _crypto.DecryptString(_file_info.GetSetFileName, _password)
            _rt_legacy_file_info.GetSetMD5FileHash = _crypto.DecryptString(_file_info.GetSetMD5FileHash, _password)

            _rt_legacy_file_info.GetSetFileSize = _file_info.GetSetFileSize

            _rt_file_list.Add(_rt_legacy_file_info)

        Next

        _rt_sfdl_file.GetSetFileInfoList = _rt_file_list

        _rt_sfdl_file.GetSetConnectionInfo.GetSetHost = _crypto.DecryptString(_legacy_sfdl_file.GetSetConnectionInfo.GetSetHost, _password)
        _rt_sfdl_file.GetSetConnectionInfo.GetSetName = _crypto.DecryptString(_legacy_sfdl_file.GetSetConnectionInfo.GetSetName, _password)
        _rt_sfdl_file.GetSetConnectionInfo.GetSetPassword = _crypto.DecryptString(_legacy_sfdl_file.GetSetConnectionInfo.GetSetPassword, _password)
        _rt_sfdl_file.GetSetConnectionInfo.GetSetPath = _crypto.DecryptString(_legacy_sfdl_file.GetSetConnectionInfo.GetSetPath, _password)
        _rt_sfdl_file.GetSetConnectionInfo.GetSetUsername = _crypto.DecryptString(_legacy_sfdl_file.GetSetConnectionInfo.GetSetUsername, _password)


        _rt_sfdl_file.GetSetConnectionInfo.GetSetPort = _legacy_sfdl_file.GetSetConnectionInfo.GetSetPort
        _rt_sfdl_file.GetSetConnectionInfo.GetSetSecurityProtocol = _legacy_sfdl_file.GetSetConnectionInfo.GetSetSecurityProtocol
        _rt_sfdl_file.GetSetConnectionInfo.GetSetTransferMode = _legacy_sfdl_file.GetSetConnectionInfo.GetSetTransferMode
        _rt_sfdl_file.GetSetConnectionInfo.GetSetTransferType = _legacy_sfdl_file.GetSetConnectionInfo.GetSetTransferType
        _rt_sfdl_file.GetSetConnectionInfo.GetSetAuthRequired = _legacy_sfdl_file.GetSetConnectionInfo.GetSetAuthRequired
        _rt_sfdl_file.GetSetConnectionInfo.GetSetCharacterEncoding = _legacy_sfdl_file.GetSetConnectionInfo.GetSetCharacterEncoding
        _rt_sfdl_file.GetSetConnectionInfo.GetSetUseSpecialCharacterFix = _rt_sfdl_file.GetSetConnectionInfo.GetSetUseSpecialCharacterFix


        Return _rt_sfdl_file

    End Function

    Private Function ConvertLegacyEncodingEnumToEncoding(ByVal _encoding As LegacyFile.Enums.CharacterEncoding) As System.Text.Encoding

        Select Case _encoding

            Case LegacyFile.Enums.CharacterEncoding.Standard

                Return System.Text.Encoding.Default

            Case LegacyFile.Enums.CharacterEncoding.UTF8

                Return System.Text.Encoding.UTF8

            Case LegacyFile.Enums.CharacterEncoding.UTF7

                Return System.Text.Encoding.UTF7

            Case LegacyFile.Enums.CharacterEncoding.ASCII

                Return System.Text.Encoding.ASCII

            Case Else

                Return System.Text.Encoding.Default

        End Select

    End Function

    Private Function ConvertLegacyFile(ByVal _legacy_file As LegacyFile.SFDL_File) As SFDLFile

        Dim _sfdl_file As New SFDLFile
        Dim _file_list As New List(Of FileInfo)
        Dim _package As New SFDLPackage

        _sfdl_file.Uploader = _legacy_file.GetsetUploader
        _sfdl_file.Description = _legacy_file.GetSetDescription
        _sfdl_file.MaxDownloadThreads = _legacy_file.GetSetMaxDownloadThread
        _sfdl_file.SFDLFileVersion = MAXSFDLFILEVERSION

        With _sfdl_file.ConnectionInfo

            .AuthRequired = _legacy_file.GetSetConnectionInfo.GetSetAuthRequired
            .CharacterEncoding = CType(_legacy_file.GetSetConnectionInfo.GetSetCharacterEncoding, CharacterEncoding.CharacterEncoding)

            If _legacy_file.GetSetConnectionInfo.GetSetTransferType = LegacyFile.SFDL.FTP.FTP.TransferType.Ascii Then
                .DataType = Net.FtpClient.FtpDataType.ASCII
            Else
                .DataType = Net.FtpClient.FtpDataType.Binary
            End If

            If _legacy_file.GetSetConnectionInfo.GetSetTransferMode = LegacyFile.SFDL.FTP.FTP.TransferMode.Active Then
                .DataConnectionType = Net.FtpClient.FtpDataConnectionType.AutoActive
            Else
                .DataConnectionType = Net.FtpClient.FtpDataConnectionType.AutoPassive
            End If

            .Host = _legacy_file.GetSetConnectionInfo.GetSetHost
            .Password = _legacy_file.GetSetConnectionInfo.GetSetPassword
            .Port = _legacy_file.GetSetConnectionInfo.GetSetPort
            .EncryptionMode = Net.FtpClient.FtpEncryptionMode.None
            .Username = _legacy_file.GetSetConnectionInfo.GetSetUsername
            .UseSpecialCharacterFix = _legacy_file.GetSetConnectionInfo.GetSetUseSpecialCharacterFix

        End With


        For Each _legacy_file_info In _legacy_file.GetSetFileInfoList

            Dim _file As New FileInfo

            _file.DirectoryPath = _legacy_file_info.GetSetDirectoryPath
            _file.DirectoryRoot = _legacy_file_info.GetSetDirectoryRoot
            _file.FileFullPath = _legacy_file_info.GetSetFileFullPath
            _file.FileName = _legacy_file_info.GetSetFileName
            _file.FileSize = _legacy_file_info.GetSetFileSize

            If String.IsNullOrEmpty(_legacy_file_info.GetSetMD5FileHash) Then
                _file.FileHashType = FileInfo.HashType.None
                _file.FileHash = String.Empty
            Else
                _file.FileHash = _legacy_file_info.GetSetMD5FileHash
                _file.FileHashType = FileInfo.HashType.MD5
            End If

            _file.PackageName = "Legacy Package"

            _file_list.Add(_file)

        Next

        _package.FileList = _file_list
        _package.Packagename = "Legacy Package"

        _sfdl_file.Packages.Add(_package)

        _sfdl_file.Encrypted = False

        Return _sfdl_file

    End Function

#End Region

#Region "SFDL Session öffnen/wiederherstellen"

    Friend Sub OpenSFDLSessions(ByVal _session_file_list() As String, ByVal _taskid As Integer)

        Dim _sfdl_session As New SFDLSession
        Dim _tools As New Core.Tools
        Dim _olv_file_item_list As New List(Of OLV_Item)

        For Each _session_file_path In _session_file_list

            _olv_file_item_list = New List(Of OLV_Item)

            _sfdl_session = CType(_tools.XMLDeSerialize(_sfdl_session, _session_file_path), SFDLSession)

            _sfdl_session.SessionState = SFDLSessionEnums.SFDLSessionState.Queued

            frm_main.AddSFDLSession(New KeyValuePair(Of String, SFDLSession)(_sfdl_session.SFDLSessionName, _sfdl_session))

            RaiseEvent AddTabPage(_sfdl_session.SFDLSessionName, _sfdl_session.SFDLFile.Description)

            For Each _package In _sfdl_session.SFDLFile.Packages

                For Each _file In _package.FileList

                    Dim _olv_file_item As New OLV_Item

                    With _olv_file_item

                        .DirectoryPath = _file.DirectoryPath
                        .DirectoryRoot = _file.DirectoryRoot
                        .FileFullPath = _file.FileFullPath
                        .FileName = _file.FileName
                        .FileSize = _file.FileSize
                        .FileHashType = _file.FileHashType
                        .FileHash = _file.FileHash
                        .PackageName = _file.PackageName
                        .Progress = 0
                        .Status = Status.Queued
                        .SFDLSessionName = _sfdl_session.SFDLSessionName

                    End With

                    _olv_file_item_list.Add(_olv_file_item)

                Next

            Next

            RaiseEvent OpenSFDLSessionComplete(_olv_file_item_list, _sfdl_session)

        Next

        RaiseEvent OpenSFDLComplete(_taskid, False)

    End Sub

#End Region

#Region "TabControl Hilfs Subs"

    Friend Function HoverTab() As TabPage

        Dim _rt As New TabPage

        For index As Int32 = 0 To frm_main.TabControlMain.TabCount - 1
            If frm_main.TabControlMain.GetTabRect(index).Contains(frm_main.TabControlMain.PointToClient(Cursor.Position)) Then
                _rt = frm_main.TabControlMain.TabPages(index)
            End If
        Next

        Return _rt

    End Function

    Public Sub SwapTabPages(ByVal tp1 As TabPage, ByVal tp2 As TabPage)
        Dim Index1 As Integer = frm_main.TabControlMain.TabPages.IndexOf(tp1)
        Dim Index2 As Integer = frm_main.TabControlMain.TabPages.IndexOf(tp2)
        frm_main.TabControlMain.TabPages(Index1) = tp2
        frm_main.TabControlMain.TabPages(Index2) = tp1
    End Sub

    Public Sub ClearEmptyTabPages()

        For Each _tab_page As TabPage In frm_main.TabControlMain.TabPages

            If GetTabOLVControl(_tab_page.Name).OLV_Package.Items.Count = 0 Then
                frm_main.RemoveSFDLSessionByKey(_tab_page.Name)
                frm_main.TabControlMain.TabPages.Remove(_tab_page)
            End If

        Next

    End Sub


    Public Sub ClearTabPages()

        For Each _tab_page As TabPage In frm_main.TabControlMain.TabPages
            frm_main.RemoveSFDLSessionByKey(_tab_page.Name)
            frm_main.TabControlMain.TabPages.Remove(_tab_page)
        Next

    End Sub

    Public Sub ClearCompletedDownloadTabs()

        For Each _tab_page As TabPage In frm_main.TabControlMain.TabPages

            If frm_main.GetSFDLSessionByKey(_tab_page.Name).SessionState = SFDLSessionEnums.SFDLSessionState.DownloadComplete Then
                frm_main.RemoveSFDLSessionByKey(_tab_page.Name)
                frm_main.TabControlMain.TabPages.Remove(_tab_page)
            End If

        Next

    End Sub

#End Region

    Friend Function GetTabOLVControl(ByVal _key As String) As Ctr_TabPage

        Dim _ctr_tab_page As New Ctr_TabPage
        Dim _index As Integer

        _index = frm_main.TabControlMain.TabPages.IndexOfKey(_key)

        _ctr_tab_page = CType(frm_main.TabControlMain.TabPages.Item(_index).Controls(0), Ctr_TabPage)

        Return _ctr_tab_page

    End Function

    Friend Function GetPackageByName(ByVal _sfdl_session As SFDLSession, ByVal _package_name As String) As SFDLPackage

        Dim _rt As New SFDLPackage

        For Each _package In _sfdl_session.SFDLFile.Packages

            If _package.Packagename.Equals(_package_name) Then
                _rt = _package
            End If
        Next

        If _rt Is Nothing Then
            Throw New Exception("Kein Package gefunden")
        Else
            Return _rt
        End If

    End Function

    Friend Function GetDownloadFilePath(ByVal _settings As Settings, ByVal _sfdl_session As SFDLSession, ByVal _olv_item As OLV_Item) As String

        Dim _download_dir As String = String.Empty
        Dim _tmp_last_sub_dir As String = String.Empty
        Dim _dowload_local_filename As String = String.Empty

        Try

            If _settings.CreateDownloadDir Then

                If _settings.CreateDownloadDirUseDescription = True And String.IsNullOrEmpty(_sfdl_session.SFDLFile.Description) = False Then
                    _download_dir = IO.Path.Combine(_settings.DownloadDirectory, _sfdl_session.SFDLFile.Description)
                Else
                    _download_dir = IO.Path.Combine(_settings.DownloadDirectory, _sfdl_session.SFDLFileName)
                End If

            Else
                _download_dir = _settings.DownloadDirectory
            End If


            If Not String.IsNullOrEmpty(_olv_item.DirectoryRoot) Then

                _tmp_last_sub_dir = _olv_item.DirectoryPath.Replace(_olv_item.DirectoryRoot, "")

                If Not String.IsNullOrEmpty(_tmp_last_sub_dir) Then
                    _tmp_last_sub_dir = _tmp_last_sub_dir.Replace("/", "\")

                    If _tmp_last_sub_dir.StartsWith("\") Then
                        _tmp_last_sub_dir = _tmp_last_sub_dir.Remove(0, 1)
                    End If

                    _download_dir = IO.Path.Combine(_download_dir, _tmp_last_sub_dir)

                End If

            End If

            If _settings.CreatePackageSubfolder = True And String.IsNullOrEmpty(_olv_item.PackageName) = False Then
                _download_dir = IO.Path.Combine(_download_dir, _olv_item.PackageName)
            End If

            If Not IO.Directory.Exists(_download_dir) Then
                IO.Directory.CreateDirectory(_download_dir)
            End If

            _dowload_local_filename = IO.Path.Combine(_download_dir, _olv_item.FileName)

        Catch ex As Exception
            Debug.WriteLine("GetDownloadFilePath Fehler")
        End Try

        Return _dowload_local_filename


    End Function

    Public Sub RefreshSFDLInfoIcon(ByVal _tabname As String)

        Dim _sfdl_session As SFDLSession

        frm_main.tsdbb_server_information.DropDownItems.Clear()

        _sfdl_session = frm_main.GetSFDLSessionByKey(_tabname)

        If Not String.IsNullOrEmpty(_sfdl_session.SFDLFile.Description) Then
            frm_main.tsdbb_server_information.DropDownItems.Add("Beschreibung: " & _sfdl_session.SFDLFile.Description, My.Resources.description)
        End If

        If Not String.IsNullOrEmpty(_sfdl_session.SFDLFile.Uploader) Then
            frm_main.tsdbb_server_information.DropDownItems.Add("Uploader: " & _sfdl_session.SFDLFile.Uploader, My.Resources.uploader)
        End If

        'Wohis Abfrage
        If frm_main.bgw_whois.IsBusy = False Then
            frm_main.bgw_whois.RunWorkerAsync(_sfdl_session.SFDLFile.ConnectionInfo.Host)
        End If

    End Sub

    Public Function ProgressClicknLoadPlugins(ByVal _url As String) As String

        Dim _tmp_param_str As String = String.Empty
        Dim _file As String = String.Empty
        Dim _file_list As New List(Of FileInfo)

        _tmp_param_str = _url

        _tmp_param_str = _tmp_param_str.Replace("sfdl://", "") 'URI Handler Prefix entfernen

        If _tmp_param_str.Contains("http://") Or _tmp_param_str.Contains("www.") Then 'handelt sich um eine URL --> ClicknLoad

            Try

                For Each _plugin As PluginInterface.PluginInterface.SFDLLoaderPlugin In frm_main.GetPluginConnectorInstance.Plugins

                    Try

                        If _plugin.SpecialPluginFunctions.Contains(PluginInterface.PluginInterface.SpeicalPluginFunction.ClicknLoad) Then

                            If _plugin.ClicknLoadURLSupported(_tmp_param_str) Then

                                _file = _plugin.ClicknLoadDownloadSFDL(_tmp_param_str)

                                Throw New PluginFoundException("Plugin gefunden!")

                            End If

                        End If

                    Catch ex As Exception
                        Debug.WriteLine("Fehler: Plugin hat einen Fehler verursacht!")
                    End Try

                Next

            Catch ex As PluginFoundException
            End Try

        End If

        Return _file

    End Function


    Public Sub ProgressCommandButtonPlugins()

        For Each _plugin As PluginInterface.PluginInterface.SFDLLoaderPlugin In frm_main.GetPluginConnectorInstance.Plugins

            Try

                If _plugin.SpecialPluginFunctions.Contains(PluginInterface.PluginInterface.SpeicalPluginFunction.CommandButton) Then

                    Dim _toolstrip_item As ToolStripItem

                    _toolstrip_item = frm_main.ExtrasToolStripMenuItem.DropDownItems.Add(_plugin.GetCommandButtonName)

                    AddHandler _toolstrip_item.Click, AddressOf _plugin.CommandButtonEvent

                End If

            Catch ex As Exception
                Debug.WriteLine("Fehler: Plugin hat einen Fehler verursacht!")
            End Try

        Next

    End Sub


End Module
