﻿Imports System.Threading
Imports System.Net.FtpClient
Imports System.IO
Imports SFDL.Core
Imports SFDL.Loader.OLV_ItemEnums

Public Class ThreadHandler

    Private Shared _stop_download_reset_event As New ManualResetEvent(False)
    Private Shared _reset_event As New ManualResetEvent(False)
    Private Shared _max_threads As Integer = 0
    Private Shared _list_ftp_connection As Dictionary(Of Guid, FTPThreadInfo)
    Private Shared _obj_lock_ftp_list As New Object
    Private Shared _obj_lock_start_download As New Object
    Private Shared _master_ftp_client As System.Net.FtpClient.FtpClient
    Private Shared _logger As New Logger

    Protected Friend Shared Event PrepareItem(ByVal _item As OLV_Item, ByVal _session_name As String)
    Protected Friend Shared Event UpdateItem(ByVal _item As OLV_Item, ByVal _session_name As String, ByVal _event_type As ThreadHandlerEnums.UpdateItemEventType)
    Protected Friend Shared Event UpdateItemProgress(ByVal _item As OLV_Item, ByVal _session_name As String, ByVal _download_speed As String, ByVal _precent_downloaded As Integer)
    Protected Friend Shared Event PrepareDownloadSFDLSessionProperty(ByVal _current_thread_count As Integer)
    Protected Friend Shared Event StopDownloadComplete()
    Protected Friend Shared Event DownloadCompleted(ByVal _connection_error As Boolean, ByVal _session_name As String)

    Protected Friend Shared Event UpdateBulkSession(ByVal _olv_list As List(Of OLV_Item), ByVal _session_name As String, ByVal _packagename As String)
    Protected Friend Shared Event BulkSessionComplete(ByVal _error As Boolean, ByVal _taskid As Integer)

    Protected Friend Sub CheckForNewDownloadThreads()
        StartDownloads()
    End Sub

    Protected Friend Sub HandleThreads()

        _list_ftp_connection = New Dictionary(Of Guid, FTPThreadInfo)

        _master_ftp_client = New FtpClient

        _logger = frm_main.GetLoggerInstance

        StartDownloads()

    End Sub

    Private Shared Sub StartDownloads()

        Dim _var_sfdl_session As New SFDLSession

        Try

            Monitor.Enter(_obj_lock_start_download)

            If frm_main.GetDownloadStopInProgress = False Then

                _logger.WriteLog("StartDownloads", "OK - Download wird gerade nicht gestoppt")

                RaiseEvent PrepareDownloadSFDLSessionProperty(_list_ftp_connection.Count)

                _var_sfdl_session = frm_main.GetSFDLDownloadSession

                If _var_sfdl_session.Items2Download.Count = 0 And _list_ftp_connection.Count = 0 Then
                    _logger.WriteLog("StartDownloads", "Session " & _var_sfdl_session.SFDLSessionName & " hat keine ausgewählten Files!")
                    Throw New DownloadCompletedException
                End If

                If _master_ftp_client.IsConnected = False Or Not _master_ftp_client.Host = _var_sfdl_session.SFDLFile.ConnectionInfo.Host Then

                    _master_ftp_client = New FtpClient

                    _logger.WriteLog("StartDownloads", "Master FTP Connection ist nicht verbunden oder der Host hat sich geändert!", LogType.Warnung)

                    _logger.WriteLog("StartDownloads", "Setzte Verbindungseinstellungen....")

                    With _master_ftp_client

                        If _var_sfdl_session.SFDLFile.ConnectionInfo.AuthRequired Then
                            .Credentials = New Net.NetworkCredential(_var_sfdl_session.SFDLFile.ConnectionInfo.Username, _var_sfdl_session.SFDLFile.ConnectionInfo.Password)
                        Else
                            .Credentials = New Net.NetworkCredential("anonymous", "anonymous@sfdl.net")
                        End If

                        .DataConnectionType = _var_sfdl_session.SFDLFile.ConnectionInfo.DataConnectionType
                        .EncryptionMode = _var_sfdl_session.SFDLFile.ConnectionInfo.EncryptionMode
                        .Host = _var_sfdl_session.SFDLFile.ConnectionInfo.Host
                        .Port = _var_sfdl_session.SFDLFile.ConnectionInfo.Port
                        .DataConnectionType = _var_sfdl_session.SFDLFile.ConnectionInfo.DataConnectionType

                        If Not .EncryptionMode = FtpEncryptionMode.None Then 'FTPS Workaround
                            .DataConnectionEncryption = False
                        End If

                    End With

                    _logger.WriteLog("StartDownloads", "Verbindungseinstellungen gesetzt!")

                    AddHandler _master_ftp_client.ValidateCertificate, AddressOf ValidateCertificateEvent

                    _logger.WriteLog("StartDownloads", "Baue nun die verbindung auf...")

                    _master_ftp_client.Connect()

                    If _master_ftp_client.IsConnected Then
                        _logger.WriteLog("StartDownloads", "Verbindung hergestellt!")
                    Else
                        Throw New Exception("Verbindung konnte nicht hergestellt werden!")
                    End If

                Else
                    _logger.WriteLog("StartDownloads", "FTP Client ist bereits verbunden!")
                End If


                For Each _item In _var_sfdl_session.Items2Download

                    Dim _guid As Guid
                    Dim _ftp_thread_info As New FTPThreadInfo
                    Dim _local_file_path As String = String.Empty
                    Dim _local_file_info As IO.FileInfo
                    Dim _ftp_async_object As New FTPAsyncObject

                    Try

                        RaiseEvent PrepareItem(_item, _var_sfdl_session.SFDLSessionName)

                        _guid = Guid.NewGuid
                        _ftp_thread_info.GUID = _guid
                        _ftp_thread_info.RetryCount = 0
                        _ftp_thread_info.FileItem = _item
                        _ftp_thread_info.SFDLSession = _var_sfdl_session

                        Monitor.Enter(_obj_lock_ftp_list)
                        _list_ftp_connection.Add(_guid, _ftp_thread_info)
                        Monitor.Exit(_obj_lock_ftp_list)

                        _logger.WriteLog("StartDownloads", "Itemname: " & _item.FileName)
                        _logger.WriteLog("StartDownloads", "GUID (Thread Info): " & _ftp_thread_info.GUID.ToString)


                        _ftp_async_object.FTPClient = _master_ftp_client
                        _ftp_async_object.FTPThreadInfo = _ftp_thread_info


                        _local_file_path = GetDownloadFilePath(frm_main.GetSettings, _var_sfdl_session, _item)

                        If IO.File.Exists(_local_file_path) = True And frm_main.GetSettings.ExistingFileHandling = ExistingFileHandling.ResumeFile Then
                            _logger.WriteLog("StartDownloads", "Lokale Datei existiert - Resume Mode")
                            _local_file_info = New FileInfo(_local_file_path)
                            _master_ftp_client.BeginOpenRead(_item.FileFullPath, _local_file_info.Length, New AsyncCallback(AddressOf OpenReadCallback), _ftp_async_object)
                        Else
                            _master_ftp_client.BeginOpenRead(_item.FileFullPath, New AsyncCallback(AddressOf OpenReadCallback), _ftp_async_object)
                        End If

                        System.Threading.Thread.Sleep(500)

                    Catch ex As FtpCommandException

                        _logger.WriteLog("StartDownloads", "Konnte Download für Item nicht starten", LogType.Fehler)
                        ParseFTPCommandException(ex, _master_ftp_client, _ftp_thread_info)

                    Catch ex As FtpException

                        _logger.WriteLog("StartDownloads", "Konnte Download für Item nicht starten", LogType.Fehler)
                        ParseFTPException(ex, _master_ftp_client, _ftp_thread_info)

                    Catch ex As Exception

                        _logger.WriteLog("StartDownloads", "Konnte Download für Item nicht starten", LogType.Fehler)
                        ParseException(ex, _master_ftp_client, _ftp_thread_info)

                    End Try

                Next

            End If


        Catch ex As DownloadCompletedException

            RaiseEvent DownloadCompleted(False, _var_sfdl_session.SFDLSessionName)

        Catch ex As Exception

            _logger.WriteLog("StartDownloads", ex.Message, LogType.Fehler)

            RaiseEvent DownloadCompleted(True, _var_sfdl_session.SFDLSessionName)

        Finally
            Monitor.Exit(_obj_lock_start_download)
        End Try

    End Sub

    Protected Friend Sub StopThreads()

        Dim _guids_remove As New List(Of Guid)
        Dim _items2update As New List(Of FTPThreadInfo)
        Dim _logger As New Logger

        _stop_download_reset_event.Reset()

        Monitor.Enter(_obj_lock_ftp_list)

        _logger = frm_main.GetLoggerInstance

        _logger.WriteLog("StopThreads", "Stoppe Downloads...")

        For Each _item In _list_ftp_connection

            Try

                Dim _var_sfdl_session As SFDLSession

                Monitor.Enter(_obj_lock_ftp_list)
                _item.Value.DownloadStopped = True
                Monitor.Exit(_obj_lock_ftp_list)

                _items2update.Add(_item.Value)

                _var_sfdl_session = frm_main.GetSFDLSessionByKey(_item.Value.SFDLSession.SFDLSessionName)
                _var_sfdl_session.TimeDownloadStopped = Now
                frm_main.UpdateSFDLSession(New KeyValuePair(Of String, SFDLSession)(_var_sfdl_session.SFDLSessionName, _var_sfdl_session))

                RaiseEvent UpdateItem(_item.Value.FileItem, _item.Value.SFDLSession.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.DownloadStopped)

            Catch ex As Exception
                _logger.WriteLog("StopThreads", "Fehler beim stoppen des Downloads!" & vbNewLine & ex.Message, LogType.Fehler)
            End Try

        Next

        For Each _item In _items2update
            UpdateFTPThreadInfo(_item)
        Next

        _logger.WriteLog("StopThreads", "Downloads gestoppt")

        _stop_download_reset_event.Reset()

        _master_ftp_client.BeginDisconnect(New AsyncCallback(AddressOf StopDownloadCallback), _master_ftp_client)

        _stop_download_reset_event.WaitOne()

        _list_ftp_connection.Clear()

        Monitor.Exit(_obj_lock_ftp_list)

        RaiseEvent StopDownloadComplete()

    End Sub

    Private Shared Function GetFTPThreadInfo(ByVal _guid As Guid) As FTPThreadInfo

        If _list_ftp_connection.ContainsKey(_guid) Then
            Return _list_ftp_connection.Item(_guid)
        Else
            Return Nothing
        End If

    End Function

    Private Shared Function IsDownloadStopped(ByVal _guid As Guid) As Boolean

        If _list_ftp_connection.ContainsKey(_guid) Then
            Return _list_ftp_connection.Item(_guid).DownloadStopped
        Else
            Return Nothing
        End If

    End Function

    Private Shared Sub UpdateFTPThreadInfo(ByVal _ftp_thread_info As FTPThreadInfo)

        If _list_ftp_connection.ContainsKey(_ftp_thread_info.GUID) Then

            Monitor.Enter(_obj_lock_ftp_list)
            _list_ftp_connection.Remove(_ftp_thread_info.GUID)
            _list_ftp_connection.Add(_ftp_thread_info.GUID, _ftp_thread_info)
            Monitor.Exit(_obj_lock_ftp_list)

        End If

    End Sub

#Region "Async Callbacks"

    Private Shared Sub OpenReadCallback(ar As IAsyncResult)

        Dim _ftp_async_object As FTPAsyncObject = TryCast(ar.AsyncState, FTPAsyncObject)
        Dim _ftp_client As FtpClient = _ftp_async_object.FTPClient

        Dim _item As OLV_Item
        Dim _sfdl_session As SFDLSession
        Dim _ftp_thread_info As FTPThreadInfo = _ftp_async_object.FTPThreadInfo

        Dim _local_file_path As String
        Dim _local_hashing As New Hashing

        Dim _fs_write As FileStream = Nothing


        Try
            If _ftp_client Is Nothing Then
                Throw New InvalidOperationException("The FtpControlConnection object is null!")
            End If

            If _ftp_thread_info Is Nothing Then
                Throw New Exception("FTP Connection nicht gefunden")
            End If

            _item = _ftp_thread_info.FileItem
            _sfdl_session = _ftp_thread_info.SFDLSession

            _logger.WriteLog("OpenReadCallback", "Itemname: " & _item.FileName)
            _logger.WriteLog("OpenReadCallback", "GUID (Thread Info): " & _ftp_thread_info.GUID.ToString)

            _local_file_path = GetDownloadFilePath(frm_main.GetSettings, _sfdl_session, _item)

            If _item.FileSize = 0 Then

                Try

                    _logger.WriteLog("OpenReadCallback", "Komisch....Angegebene Item Datei Größe ist 0 - aktualisiere Dateigröße....", LogType.Warnung)
                    _item.FileSize = _ftp_client.GetFileSize(_item.FileFullPath)

                    _logger.WriteLog("OpenReadCallback", "Dateigröße erfolgreich aktualisiert!", LogType.Info)

                Catch ex As Exception
                    _logger.WriteLog("OpenReadCallback", "Dateigröße konnte leider nicht aktualisiert werden!", LogType.Warnung)
                End Try

            End If

            Using _fs_read As Stream = _ftp_client.EndOpenRead(ar)

                Try

                    Dim Length As Integer = 256
                    Dim buffer As [Byte]() = New [Byte](Length - 1) {}
                    Dim bytesRead As Integer = _fs_read.Read(buffer, 0, Length)

                    Dim _local_file_info As FileInfo
                    Dim _starttime As DateTime = DateTime.Now

                    Dim _event_counter As Integer = 0
                    Dim _flush_disk_counter As Integer = 0
                    Const EVENTRAISE As Integer = 1000

                    If IO.File.Exists(_local_file_path) = True And frm_main.GetSettings.ExistingFileHandling = ExistingFileHandling.ResumeFile Then

                        _logger.WriteLog("OpenReadCallback", "Resume Mode")

                        _local_file_info = New FileInfo(_local_file_path)

                        _fs_write = New FileStream(_local_file_path, FileMode.Append, FileAccess.Write)

                        _logger.WriteLog("OpenReadCallback", "Lokal Position: " & _fs_write.Position)
                        _logger.WriteLog("OpenReadCallback", "FTP Position: " & _fs_read.Position)

                    Else
                        _fs_write = New FileStream(_local_file_path, FileMode.Create, FileAccess.Write)
                    End If

                    ' write the required bytes
                    While bytesRead > 0 And IsDownloadStopped(_ftp_thread_info.GUID) = False

                        Dim _percent_downloaded As Double = 0
                        Dim _amount_downloaded As String = String.Empty
                        Dim _download_speed As String = String.Empty

                        If _fs_read.Length > 0 Then
                            _percent_downloaded = CDbl(_fs_read.Position) / CDbl(_fs_read.Length)
                        End If

                        _fs_write.Write(buffer, 0, bytesRead)
                        bytesRead = _fs_read.Read(buffer, 0, Length)

                        _amount_downloaded = CStr(_fs_read.Position)
                        _download_speed = (_fs_read.Position / DateTime.Now.Subtract(_starttime).TotalSeconds).FormatBytesAsKBperSecond()

                        _event_counter += 1
                        _flush_disk_counter += 1

                        _item.SizeDownloaded = CLng(_amount_downloaded)

                        If _event_counter = EVENTRAISE Then 'Nicht jedesmal Update = More Performance
                            _fs_write.Flush()
                            RaiseEvent UpdateItemProgress(_item, _sfdl_session.SFDLSessionName, _download_speed, CInt(_percent_downloaded * 100))
                            _event_counter = 0
                        End If

                    End While

                Finally

                    If Not _fs_write Is Nothing Then

                        _fs_write.Close()
                        _fs_read.Close()

                    End If


                    Try

                        If _ftp_client.HasFeature(FtpCapability.MD5) And _item.FileHashType = File.FileInfo.HashType.None Then

                            Dim _reply As FtpReply

                            _logger.WriteLog("OpenReadCallback", "Server unstützt MD5 Hash - Hole Hash")

                            _reply = _ftp_client.Execute("MD5 {0}", _item.FileFullPath)

                            If _reply.Success Then

                                Dim _tmp_hash As String = String.Empty

                                _item.FileHashType = File.FileInfo.HashType.MD5
                                _tmp_hash = _reply.Message.Replace(_item.FileFullPath, "")
                                _item.FileHash = _tmp_hash.Trim

                            End If

                        End If


                        If _ftp_client.HasFeature(FtpCapability.XMD5) And _item.FileHashType = File.FileInfo.HashType.None Then

                            Dim _reply As FtpReply

                            _logger.WriteLog("OpenReadCallback", "Server unstützt MD5 Hash - Hole Hash")

                            _reply = _ftp_client.Execute("XMD5 {0}", _item.FileFullPath)

                            If _reply.Success Then

                                Dim _tmp_hash As String = String.Empty

                                _item.FileHashType = File.FileInfo.HashType.MD5
                                _tmp_hash = _reply.Message.Replace(_item.FileFullPath, "")
                                _item.FileHash = _tmp_hash.Trim

                            End If

                        End If


                        If _ftp_client.HasFeature(FtpCapability.XSHA1) And _item.FileHashType = File.FileInfo.HashType.None Then

                            Dim _reply As FtpReply

                            _logger.WriteLog("OpenReadCallback", "Server unstützt SHA1 Hash - Hole Hash")

                            _reply = _ftp_client.Execute("XSHA1 {0}", _item.FileFullPath)

                            If _reply.Success Then

                                Dim _tmp_hash As String = String.Empty

                                _item.FileHashType = File.FileInfo.HashType.SHA1
                                _tmp_hash = _reply.Message.Replace(_item.FileFullPath, "")
                                _item.FileHash = _tmp_hash.Trim

                            End If

                        End If


                        If _ftp_client.HasFeature(FtpCapability.XCRC) And _item.FileHashType = File.FileInfo.HashType.None Then

                            Dim _reply As FtpReply

                            _logger.WriteLog("OpenReadCallback", "Server unstützt CRC Hash - Hole Hash")

                            _reply = _ftp_client.Execute("XCRC {0}", _item.FileFullPath)

                            If _reply.Success Then

                                Dim _tmp_hash As String = String.Empty

                                _item.FileHashType = File.FileInfo.HashType.CRC
                                _tmp_hash = _reply.Message.Replace(_item.FileFullPath, "")
                                _item.FileHash = _tmp_hash.Trim

                            End If

                        End If

                    Catch ex As FtpCommandException
                        _logger.WriteLog("OpenReadCallback", "Hash konnte FTP Seitig nicht ermittelt werden!", LogType.Fehler)
                        _item.FileHashType = File.FileInfo.HashType.None
                        _item.FileHash = String.Empty
                    End Try




                    Select Case _item.FileHashType

                        Case File.FileInfo.HashType.None

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.DownloadComplete)

                        Case File.FileInfo.HashType.MD5

                            _logger.WriteLog("OpenReadCallback", "MD5 Hash vorhanden - Prüfe")

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.FileValdilation)

                            If _local_hashing.MD5FileHash(_local_file_path).ToLower.Equals(_item.FileHash.ToString.ToLower) Then
                                _logger.WriteLog("OpenReadCallback", "MD5 Hash Valid")
                                RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.DownloadComplete_HashValid)
                            Else
                                _logger.WriteLog("OpenReadCallback", "MD5 Hash invalid", LogType.Warnung)
                                RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.DownloadComplete_HashInvalid)
                            End If

                        Case File.FileInfo.HashType.CRC

                            _logger.WriteLog("OpenReadCallback", "CRC Checksumme vorhanden - Prüfe")

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.FileValdilation)

                            If _local_hashing.CRC32FileHash(_local_file_path).ToLower.Equals(_item.FileHash.ToString.ToLower) Then
                                _logger.WriteLog("OpenReadCallback", "CRC Checksumme Valid")
                                RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.DownloadComplete_HashValid)
                            Else
                                _logger.WriteLog("OpenReadCallback", "CRC Checksumme Invalid", LogType.Warnung)
                                RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.DownloadComplete_HashInvalid)
                            End If


                        Case File.FileInfo.HashType.SHA1


                            Debug.WriteLine("SHA1 Hash vorhanden - Prüfe")

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.FileValdilation)

                            If _local_hashing.SHA1FileHash(_local_file_path).ToLower.Equals(_item.FileHash.ToString.ToLower) Then
                                _logger.WriteLog("OpenReadCallback", "SHA1 Hash Valid")
                                RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.DownloadComplete_HashValid)
                            Else
                                _logger.WriteLog("OpenReadCallback", "SHA1 Hash invalid", LogType.Warnung)
                                RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.DownloadComplete_HashInvalid)
                            End If


                    End Select


                    If _list_ftp_connection.ContainsKey(_ftp_thread_info.GUID) Then

                        Monitor.Enter(_obj_lock_ftp_list)
                        _list_ftp_connection.Remove(_ftp_thread_info.GUID)
                        Monitor.Exit(_obj_lock_ftp_list)
                        _logger.WriteLog("OpenReadCallback", "FTP Client mit der GUID " & _ftp_thread_info.GUID.ToString & " aus der Liste entfernt!")

                    End If

                    System.Threading.Thread.Sleep(1000)

                    If _list_ftp_connection.Count = 0 Then
                        _logger.WriteLog("OpenReadCallback", "FTP Connection Liste ist leer - Trenne Verbindung für diese Session!")
                        _ftp_client.BeginDisconnect(New AsyncCallback(AddressOf DisconnectCallback), _ftp_async_object)
                    End If

                    StartDownloads()


                End Try

            End Using

        Catch ex As FtpCommandException

            _logger.WriteLog("OpenReadCallback", "[FTPCommand] " & ex.Message, LogType.Fehler)

            If _ftp_thread_info.DownloadStopped = False Then ' Event nur werfen wenn der Download nicht gestoppt ist
                ParseFTPCommandException(ex, _ftp_client, _ftp_thread_info)
            End If

        Catch ex As FtpException

            _logger.WriteLog("OpenReadCallback", "[FTP] " & ex.Message, LogType.Fehler)

            If _ftp_thread_info.DownloadStopped = False Then
                ParseFTPException(ex, _ftp_client, _ftp_thread_info)
            End If

        Catch ex As Exception

            _logger.WriteLog("OpenReadCallback", ex.Message, LogType.Fehler)
            If _ftp_thread_info.DownloadStopped = False Then
                ParseException(ex, _ftp_client, _ftp_thread_info)
            End If

        Finally

            If Not _fs_write Is Nothing Then

                _fs_write.Close()
                _fs_write.Dispose()
            End If

            _reset_event.Set()

        End Try

    End Sub


    Private Shared Sub DisconnectCallback(ar As IAsyncResult)

        Dim _ftp_async_object As FTPAsyncObject = TryCast(ar.AsyncState, FTPAsyncObject)
        Dim _ftp_client As FtpClient = _ftp_async_object.FTPClient
        Dim _ftp_thread_info As FTPThreadInfo = _ftp_async_object.FTPThreadInfo
        Dim _logger As New Logger

        _logger = frm_main.GetLoggerInstance

        Try
            If _ftp_client Is Nothing Then
                Throw New InvalidOperationException("The FtpControlConnection object is null!")
            End If

            _logger.WriteLog("DisconnectCallback", "Verbindung getrennt!")

            _ftp_client.EndDisconnect(ar)

        Catch ex As FtpCommandException

            _logger.WriteLog("DisconnectCallback", "[FTPCommand] " & ex.Message, LogType.Fehler)
            ParseFTPCommandException(ex, _ftp_client, _ftp_thread_info)

        Catch ex As FtpException

            _logger.WriteLog("DisconnectCallback", "[FTP] " & ex.Message, LogType.Fehler)
            ParseFTPException(ex, _ftp_client, _ftp_thread_info)

        Catch ex As Exception

            _logger.WriteLog("DisconnectCallback", ex.Message, LogType.Fehler)
            ParseException(ex, _ftp_client, _ftp_thread_info)

        Finally

            StartDownloads()

        End Try
    End Sub

    Private Shared Sub StopDownloadCallback(ar As IAsyncResult)

        Dim _ftp_client As FtpClient = TryCast(ar.AsyncState, FtpClient)
        Dim _logger As New Logger

        _logger = frm_main.GetLoggerInstance

        Try
            If _ftp_client Is Nothing Then
                Throw New InvalidOperationException("The FtpControlConnection object is null!")
            End If

            _logger.WriteLog("StopDownloadCallback", "Verbindung getrennt!")
            _ftp_client.EndDisconnect(ar)


        Catch ex As Exception
            _logger.WriteLog("StopDownloadCallback", ex.Message, LogType.Fehler)
        Finally
            _stop_download_reset_event.Set()
        End Try
    End Sub


#End Region

#Region "Fehler Parse Funktionen"

    Private Shared Sub ParseException(ByVal exception As Exception, ByVal _ftp_client As FtpClient, ByVal _ftp_thread_info As FTPThreadInfo)

        Dim _item As OLV_Item
        Dim _sfdl_session As SFDLSession

        Try


            If _ftp_thread_info Is Nothing Then
                Throw New Exception("FTP Connection nicht gefunden")
            End If

            _item = _ftp_thread_info.FileItem
            _sfdl_session = _ftp_thread_info.SFDLSession

            'Aus der Liste enfernen
            Monitor.Enter(_obj_lock_ftp_list)
            If _list_ftp_connection.ContainsKey(_ftp_thread_info.GUID) Then
                _list_ftp_connection.Remove(_ftp_thread_info.GUID)
            End If
            Monitor.Exit(_obj_lock_ftp_list)

            'Try Disconnect

            If _list_ftp_connection.Count = 0 Then

                _logger.WriteLog("ParseException", "List FTP Verbindungen ist leer - Ich trenne mal die Verbindung...")

                Try
                    If _ftp_client.IsConnected Then
                        _ftp_client.Disconnect()
                    End If
                Catch ex_disconnect As Exception
                End Try

            End If

            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.UnknownError)

        Catch ex As Exception
            Debug.WriteLine("[ParseFTPException] " & ex.Message)
        Finally
            StartDownloads()
        End Try

    End Sub

    Private Shared Sub ParseFTPException(ByVal ex_ftp As FtpException, ByVal _ftp_client As FtpClient, ByVal _ftp_thread_info As FTPThreadInfo)

        Dim _item As OLV_Item
        Dim _sfdl_session As SFDLSession

        Try

            If _ftp_thread_info Is Nothing Then
                Throw New Exception("FTP Connection nicht gefunden")
            End If

            _item = _ftp_thread_info.FileItem
            _sfdl_session = _ftp_thread_info.SFDLSession

            'Aus der Liste enfernen
            Monitor.Enter(_obj_lock_ftp_list)
            If _list_ftp_connection.ContainsKey(_ftp_thread_info.GUID) Then
                _list_ftp_connection.Remove(_ftp_thread_info.GUID)
            End If
            Monitor.Exit(_obj_lock_ftp_list)

            'Try Disconnect

            If _list_ftp_connection.Count = 0 Then

                _logger.WriteLog("ParseFTPException", "List FTP Verbindungen ist leer - Ich trenne mal die Verbindung...")

                Try
                    If _ftp_client.IsConnected Then
                        _ftp_client.Disconnect()
                    End If
                Catch ex_disconnect As Exception
                End Try

            End If

            If ex_ftp.Message.ToLower.Contains("type set to i") Then
                RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.ConnectionError)
            Else
                RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.UnknownError)
            End If



        Catch ex As Exception
            Debug.WriteLine("[ParseFTPException] " & ex.Message)
        Finally
            StartDownloads()
        End Try


    End Sub

    Private Shared Sub ParseFTPCommandException(ByVal ex_ftp As FtpCommandException, ByVal _ftp_client As FtpClient, ByVal _ftp_thread_info As FTPThreadInfo)

        Dim _item As OLV_Item
        Dim _sfdl_session As SFDLSession
        Dim _ftp_async_object As New FTPAsyncObject

        Try

            _ftp_thread_info = GetFTPThreadInfo(_ftp_thread_info.GUID)

            If _ftp_thread_info Is Nothing Then
                Throw New Exception("FTP Connection nicht gefunden")
            End If

            _item = _ftp_thread_info.FileItem
            _sfdl_session = _ftp_thread_info.SFDLSession

            _logger.WriteLog("ParseFTPCommandException", "Itemname: " & _item.FileName)
            _logger.WriteLog("ParseFTPCommandException", "GUID (Thread Info): " & _ftp_thread_info.GUID.ToString)


            'If _list_ftp_connection.Count = 0 Then '#################### Evtl. später wieder einbauen

            '    Debug.WriteLine("List FTP Verbindungen ist leer - Ich trenne mal die Verbindung...")

            '    Try
            '        If _ftp_client.IsConnected Then
            '            _ftp_client.Disconnect()
            '        End If
            '    Catch ex_disconnect As Exception
            '    End Try

            'End If

            If _ftp_thread_info.DownloadStopped = False Then

                _logger.WriteLog("ParseFTPCommandException", "Download ist nicht gestoppt")

                If Not _ftp_thread_info.RetryCount >= frm_main.GetSettings.MaxRetry Then

                    Dim _local_file_path As String
                    Dim _local_file_info As FileInfo

                    RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.Retry)

                    _logger.WriteLog("ParseFTPCommandException", "Retry - Starte Download für dieses Item in 2 Sekunden neu")

                    System.Threading.Thread.Sleep(2000) 'Hier warten

                    _ftp_thread_info.RetryCount += 1

                    UpdateFTPThreadInfo(_ftp_thread_info)

                    _local_file_path = GetDownloadFilePath(frm_main.GetSettings, _sfdl_session, _item)

                    _ftp_async_object.FTPClient = _ftp_client
                    _ftp_async_object.FTPThreadInfo = _ftp_thread_info

                    If IO.File.Exists(_local_file_path) = True And frm_main.GetSettings.ExistingFileHandling = ExistingFileHandling.ResumeFile Then
                        _logger.WriteLog("ParseFTPCommandException", "Lokale Datei existiert - Resume Mode")
                        _local_file_info = New FileInfo(_local_file_path)
                        _ftp_client.BeginOpenRead(_item.FileFullPath, _local_file_info.Length, New AsyncCallback(AddressOf OpenReadCallback), _ftp_async_object)
                    Else
                        _ftp_client.BeginOpenRead(_item.FileFullPath, New AsyncCallback(AddressOf OpenReadCallback), _ftp_async_object)
                    End If

                Else

                    _logger.WriteLog("ParseFTPCommandException", "Retry Maximum bereits ereicht!")

                    'Aus der Liste enfernen
                    Monitor.Enter(_obj_lock_ftp_list)
                    If _list_ftp_connection.ContainsKey(_ftp_thread_info.GUID) Then
                        _list_ftp_connection.Remove(_ftp_thread_info.GUID)
                    End If
                    Monitor.Exit(_obj_lock_ftp_list)

                    Select Case ex_ftp.CompletionCode

                        Case "421" ' Service not available, closing control connection. This may be a reply to any command if the service knows it must shut down.

                            If ex_ftp.Message.ToString.ToLower.Contains("maximum login limit has been reached.") Then
                                RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.ServerFull)
                            Else
                                RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.ServerDown)
                            End If

                        Case "425" ' Can't open data connection.

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.ConnectionError)

                        Case "426" 'Connection closed; transfer aborted.

                            'Download Stopped

                        Case "430" ' Invalid username or password

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.AuthError)

                        Case "434" 'Requested host unavailable.

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.ServerDown)

                        Case "450" 'Requested file action not taken.

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.FileNotFound)

                        Case "451" 'Requested action aborted. Local error in processing

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.InternalServerError)

                        Case "452"

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.InternalServerError)

                        Case "501" 'Syntax error in parameters or arguments.

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.DirectoryNotFound)

                        Case "502" 'Command not implemented.

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.InternalServerError)

                        Case "503" 'Bad sequence of commands.

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.InternalServerError)

                        Case "504" 'Command not implemented for that parameter

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.InternalServerError)

                        Case "530" 'Not logged in.

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.AuthError)

                        Case "550" 'Requested action not taken. File unavailable (e.g., file not found, no access).

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.FileNotFound)

                        Case "553" ' Requested action not taken. File name not allowed.

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.FileNotFound)

                        Case Else

                            RaiseEvent UpdateItem(_item, _sfdl_session.SFDLSessionName, ThreadHandlerEnums.UpdateItemEventType.UnknownError)

                    End Select


                End If

            End If

        Catch ex As Exception
            Debug.WriteLine("[ParseFTPCommandException] " & ex.Message)
        Finally
            StartDownloads()
        End Try

    End Sub




#End Region

#Region "Bulk Folder Threads"

    Protected Friend Sub GetBulkFileList(ByVal _package As SFDL.File.SFDLPackage, ByVal _sdfl_connection_info As SFDL.File.ConnectionInfo, ByVal _session_name As String, ByVal _taskid As Integer)

        Dim _ftp_client As New FtpClient
        Dim _error As Boolean = False

        Try

            With _ftp_client

                If _sdfl_connection_info.AuthRequired Then
                    .Credentials = New Net.NetworkCredential(_sdfl_connection_info.Username, _sdfl_connection_info.Password)
                Else
                    .Credentials = New Net.NetworkCredential("anonymous", "anonymous@sfdl.net")
                End If

                .DataConnectionType = _sdfl_connection_info.DataConnectionType
                .EncryptionMode = _sdfl_connection_info.EncryptionMode
                .Host = _sdfl_connection_info.Host
                .Port = _sdfl_connection_info.Port
                .DataConnectionType = _sdfl_connection_info.DataConnectionType

            End With

            AddHandler _ftp_client.ValidateCertificate, AddressOf ValidateCertificateEvent

            _ftp_client.Connect()

            For Each _bulk_folder In _package.BulkFolderList

                If GetListing(_bulk_folder.BulkFolderPath, _ftp_client, _session_name, _bulk_folder) = True Then 'Wen True ist ein Fehler aufgetreten
                    _error = True
                    MessageBox.Show("Es ist ein Fehler ermitteln der Dateien aufgetreten - SFDL konnte nicht geöffnet werden!", "Bulk Folder Downlad Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
                End If

            Next

            _ftp_client.Disconnect()

        Catch ex As Exception
            _error = True
        Finally
            RaiseEvent BulkSessionComplete(_error, _taskid)
        End Try


    End Sub

    Private Shared Function GetListing(ByVal _path As String, ByVal _ftp_client As FtpClient, ByVal _sfdl_session_name As String, ByVal _bulk_folder As SFDL.File.BulkFolder) As Boolean

        Dim _olv_file_list As New List(Of OLV_Item)
        Dim _rt_error_occured As Boolean = False
        Dim _logger As New Logger

        Try

            _logger = frm_main.GetLoggerInstance

            For Each _item As FtpListItem In _ftp_client.GetListing(_path)

                If _item.Type = FtpFileSystemObjectType.Directory Then
                    GetListing(_item.FullName, _ftp_client, _sfdl_session_name, _bulk_folder)
                Else

                    If _item.Type = FtpFileSystemObjectType.File Then

                        _logger.WriteLog("GetListing", "Dateiname : " & _item.Name)
                        _logger.WriteLog("GetListing", "Dateipfad: " & _item.FullName)

                        Dim _olv_item As New OLV_Item

                        With _olv_item

                            .DirectoryRoot = _bulk_folder.BulkFolderPath
                            .DirectoryPath = IO.Path.GetDirectoryName(_item.FullName)
                            .FileFullPath = _item.FullName
                            .FileName = _item.Name
                            .FileSize = _item.Size
                            .Progress = 0
                            .SizeDownloaded = 0
                            .PackageName = _bulk_folder.PackageName
                            .Status = Status.Queued
                            .StatusText = String.Empty
                            .DownloadSpeed = String.Empty
                            .FileHash = String.Empty
                            .FileHashType = File.FileInfo.HashType.None
                            .SFDLSessionName = _sfdl_session_name

                        End With

                        _olv_file_list.Add(_olv_item)

                    End If

                End If

            Next

            If _olv_file_list.Count = 0 Then
                _rt_error_occured = True
            End If

            RaiseEvent UpdateBulkSession(_olv_file_list, _sfdl_session_name, _bulk_folder.PackageName)


        Catch ex As FtpCommandException

            _logger.WriteLog("GetListing", ex.Message, LogType.Fehler)
            _rt_error_occured = True

        Catch ex As FtpException

            _logger.WriteLog("GetListing", ex.Message, LogType.Fehler)
            _rt_error_occured = True

        Catch ex As Exception

            _logger.WriteLog("GetListing", ex.Message, LogType.Fehler)
            _rt_error_occured = True

        Finally
            _reset_event.Set()
        End Try

        Return _rt_error_occured

    End Function

#End Region

    Private Shared Sub ValidateCertificateEvent(control As FtpClient, e As FtpSslValidationEventArgs)
        e.Accept = True
    End Sub




End Class
