﻿Imports SFDL.Core
Imports SFDL.File
Imports BrightIdeasSoftware
Imports System.Threading
Imports SFDL.Loader.OLV_ItemEnums
Imports System.Text

Public Class frm_main

    Private Shared CONST_PLUGINDIR As String = IO.Path.Combine(Application.StartupPath, "plugins")
    Private Shared CONST_SETTINGSFILEPATH As String = String.Empty

    Private Shared _var_sfdl_session_list As Dictionary(Of String, SFDLSession)
    Private Shared _var_item_speed_size_collection As New Dictionary(Of String, OLV_Item)
    Private Shared _var_sfdl_download_session As SFDLSession
    Private Shared _var_download_running As Boolean = False
    Private Shared _var_download_stop_in_progress As Boolean = False

    Private Shared _var_remaining_time As String = String.Empty
    Private Shared _var_download_speed As String = String.Empty
    Private Shared _var_download_progress As String = String.Empty


    Private DragStartPosition As Point = Point.Empty
    Private _obj_lock_SizeSpeedCollection As New Object
    Private _obj_lock_PrepareDownloadSFDLSessionProperty As New Object
    Private _obj_lock_SFDLSessionList As New Object
    Private _obj_lock_Task As New Object

    Private Shared _logger As New Logger
    Private Shared _settings As Settings
    Private Shared _plugin_connector As New PluginConnector
    Private Shared _cleanup_timer As Timer
    Private _tools As New Tools

#Region "Public ReadOnly Properties"

    Public ReadOnly Property GetSettings As Settings
        Get
            Return _settings
        End Get
    End Property

    Public ReadOnly Property GetSettingsPath As String
        Get
            Return CONST_SETTINGSFILEPATH
        End Get
    End Property

    Public ReadOnly Property GetSFDLSessionStartTimeByKey(ByVal _session_uid As String) As Date
        Get

            Dim _rt As Date = Nothing

            If _var_sfdl_session_list.ContainsKey(_session_uid) Then
                _rt = _var_sfdl_session_list(_session_uid).TimeDownloadStarted
            End If

            Return _rt

        End Get
    End Property

    Public ReadOnly Property GetSFDLSessionStopTimeByKey(ByVal _session_uid As String) As Date
        Get

            Dim _rt As Date = Nothing

            If _var_sfdl_session_list.ContainsKey(_session_uid) Then
                _rt = _var_sfdl_session_list(_session_uid).TimeDownloadStopped
            End If

            Return _rt

        End Get
    End Property

    Public ReadOnly Property GetSFDLDescriptionByKey(ByVal _session_uid As String) As String
        Get

            Dim _rt As String = String.Empty

            If _var_sfdl_session_list.ContainsKey(_session_uid) Then
                _rt = _var_sfdl_session_list(_session_uid).SFDLFile.Description
            End If

            Return _rt

        End Get
    End Property


    Public ReadOnly Property GetSFDLUploaderByKey(ByVal _session_uid As String) As String
        Get

            Dim _rt As String = String.Empty

            If _var_sfdl_session_list.ContainsKey(_session_uid) Then
                _rt = _var_sfdl_session_list(_session_uid).SFDLFile.Uploader
            End If

            Return _rt

        End Get
    End Property

    Public ReadOnly Property GetSFDLSessionPackageListByKey(ByVal _session_uid As String) As List(Of SFDLPackage)
        Get

            Dim _rt As New List(Of SFDLPackage)

            If _var_sfdl_session_list.ContainsKey(_session_uid) Then
                _rt = _var_sfdl_session_list(_session_uid).SFDLFile.Packages
            End If

            Return _rt

        End Get
    End Property

    Public ReadOnly Property GetSFDLSessionCheckedItemsByKey(ByVal _session_uid As String) As List(Of OLV_Item)
        Get

            Dim _rt As New List(Of OLV_Item)

            If _var_sfdl_session_list.ContainsKey(_session_uid) Then
                _rt = _var_sfdl_session_list(_session_uid).CheckedItems
            End If

            Return _rt

        End Get
    End Property

    Public ReadOnly Property GetLoggerInstance As Logger
        Get
            Return _logger
        End Get
    End Property


#End Region

#Region "Private ReadOnly Properties"


    Protected Friend ReadOnly Property GetPluginsPath As String
        Get
            Return CONST_PLUGINDIR
        End Get
    End Property

    Protected Friend ReadOnly Property GetSFDLSessionByKey(ByVal _key As String) As SFDLSession
        Get
            If _var_sfdl_session_list.ContainsKey(_key) Then
                Return _var_sfdl_session_list.Item(_key)
            Else
                Return Nothing
            End If
        End Get
    End Property

    Protected Friend ReadOnly Property GetSFDLSessionList As Dictionary(Of String, SFDLSession)
        Get
            Return _var_sfdl_session_list
        End Get
    End Property

    Protected Friend ReadOnly Property GetSFDLDownloadSession As SFDLSession
        Get
            Return _var_sfdl_download_session
        End Get
    End Property

    Protected Friend ReadOnly Property GetDownloadRunning As Boolean
        Get
            Return _var_download_running
        End Get
    End Property

    Protected Friend ReadOnly Property GetDownloadStopInProgress As Boolean
        Get
            Return _var_download_stop_in_progress
        End Get
    End Property

    Protected Friend ReadOnly Property GetPluginConnectorInstance As PluginConnector
        Get
            Return _plugin_connector
        End Get
    End Property

#End Region

#Region "Event Handler"

    Private Sub CleanUpTimer(ByVal _state As Object)

        Dim _index As Integer = CInt(_state)

        If Me.InvokeRequired Then
            Me.Invoke(Sub() Me.CleanUpTimer(_state))

        Else

            Try

                Monitor.Enter(_obj_lock_Task)

                StatusStripTasks.Items.RemoveAt(_index)

                If StatusStripTasks.Items.Count = 0 Then
                    StatusStripTasks.Visible = False
                End If

            Catch ex As Exception

            Finally
                Monitor.Exit(_obj_lock_Task)
            End Try


        End If

    End Sub

    Private Sub frm_setting_settings_update_event(_new_settings As Settings)
        _settings = _new_settings
    End Sub

    'Private Sub PluginsLoaded(_plugins As List(Of PluginInterface.PluginInterface.SFDLLoaderPlugin))

    '    If Me.InvokeRequired Then
    '        Me.Invoke(Sub() Me.PluginsLoaded(_plugins))
    '    Else
    '        _var_plugin_list = _plugins
    '        Debug.WriteLine("{0} Plugins geladen!", _var_plugin_list.Count)
    '    End If

    'End Sub

    Private Sub PrepareDownloadSFDLSessionPropertyEvent(ByVal _thread_counter As Integer)

        Dim _var_sfdl_session As New SFDLSession
        Dim _list_checked As New List(Of OLV_Item)


        If Me.InvokeRequired Then
            Me.Invoke(Sub() Me.PrepareDownloadSFDLSessionPropertyEvent(_thread_counter))
        Else

            Try

                For Each _tab As TabPage In TabControlMain.TabPages

                    Dim _olv_control As New Ctr_TabPage
                    Dim _max_threads As Integer = 0

                    _olv_control = GetTabOLVControl(_tab.Name)
                    _var_sfdl_session = GetSFDLSessionByKey(_tab.Name)

                    If _settings.MaxDownloadThreads < _var_sfdl_session.SFDLFile.MaxDownloadThreads Then
                        _max_threads = _settings.MaxDownloadThreads
                    Else
                        _max_threads = _var_sfdl_session.SFDLFile.MaxDownloadThreads
                    End If

                    Debug.WriteLine("Ermittle Maximale Thread Anzahl:" & _max_threads)

                    _var_sfdl_session.Items2Download = New List(Of OLV_Item) 'Liste zurücksetzten

                    If _thread_counter = _max_threads Then
                        Debug.WriteLine("Maximale Anzahl ist bereits erreicht!")
                        Throw New MaxThreadCounterReached
                    End If


                    For Each _item As OLV_Item In _olv_control.OLV_Package.CheckedObjects

                        Debug.WriteLine("Item Name: " & _item.FileName)
                        Debug.WriteLine("Item Status: " & _item.Status.ToString)

                        Select Case _item.Status

                            Case Status.Waiting

                                _list_checked.Add(_item)
                                _thread_counter += 1
                                Debug.WriteLine("Thread Counter um 1 erhöt!")

                        End Select


                        If _thread_counter = _max_threads Then

                            Debug.WriteLine("Thread Counter hat Max Threads erreicht - gebe SFDL Session mit Items an Thread Handler zurück!")

                            If _var_sfdl_session.SessionState = SFDLSessionEnums.SFDLSessionState.Queued Then
                                _var_sfdl_session.TimeDownloadStarted = Now
                                UpdateSFDLSession(New KeyValuePair(Of String, SFDLSession)(_var_sfdl_session.SFDLSessionName, _var_sfdl_session))
                            End If

                            SetSFDLSessionState(_var_sfdl_session.SFDLSessionName, SFDLSessionEnums.SFDLSessionState.DownloadRunning)
                            _var_sfdl_session.SessionState = SFDLSessionEnums.SFDLSessionState.DownloadRunning

                            Throw New MaxThreadCounterReached

                        End If

                    Next

                    Debug.WriteLine("Max Thread Anzahl nicht erreicht -  Es wurden nur " & _list_checked.Count & " gefunden.")

                    If _var_sfdl_session.SessionState = SFDLSessionEnums.SFDLSessionState.Queued Then
                        _var_sfdl_session.TimeDownloadStarted = Now
                        UpdateSFDLSession(New KeyValuePair(Of String, SFDLSession)(_var_sfdl_session.SFDLSessionName, _var_sfdl_session))
                    End If

                    If Not _thread_counter = 0 Then
                        Throw New MaxThreadCounterReached
                    Else
                        DownloadCompleteEvent(False, String.Empty) 'Somit wird gepürft ob nicht eine Session fertig ist
                        Debug.WriteLine("Thread Counter ist null - werfe kein event damit das nächste Tab abgearbeitet wird!")
                    End If

                Next

            Catch ex As MaxThreadCounterReached

            Catch ex As Exception

                Debug.WriteLine("Fehler: " & ex.Message)

            Finally

                _var_sfdl_session.Items2Download = _list_checked
                _var_sfdl_download_session = _var_sfdl_session

            End Try

        End If

    End Sub

    Private Sub PrepareItemEvent(_item As OLV_Item, _session_name As String)

        Try

            If Me.InvokeRequired Then
                Me.Invoke(Sub() Me.PrepareItemEvent(_item, _session_name))
            Else

                Dim _index As Integer
                Dim _index_item As Integer
                Dim _object As Ctr_TabPage

                _index = TabControlMain.TabPages.IndexOfKey(_session_name)

                _object = CType(TabControlMain.TabPages.Item(_index).Controls(0), Ctr_TabPage)

                _index_item = _object.OLV_Package.IndexOf(_item)

                _item.Status = Status.Waiting
                _item.StatusText = "Warte..."
                _item.Progress = 0

                _object.OLV_Package.RefreshItem(_object.OLV_Package.GetItem(_index_item))

            End If

        Catch ex As Exception
            Debug.WriteLine("Fehler! Konnte Item nicht preparieren!")
        End Try

    End Sub

    Private Sub UpdateItemEvent(_item As OLV_Item, _session_name As String, _event_type As ThreadHandlerEnums.UpdateItemEventType)

        Try

            If Me.InvokeRequired Then
                Me.Invoke(Sub() Me.UpdateItemEvent(_item, _session_name, _event_type))
            Else

                Dim _index As Integer
                Dim _index_item As Integer
                Dim _object As Ctr_TabPage

                _index = TabControlMain.TabPages.IndexOfKey(_session_name)

                _object = CType(TabControlMain.TabPages.Item(_index).Controls(0), Ctr_TabPage)

                _index_item = _object.OLV_Package.IndexOf(_item)


                Select Case _event_type

                    Case ThreadHandlerEnums.UpdateItemEventType.DirectoryNotFound

                        _item.Status = Status.Failure
                        _item.StatusText = "Verzeichnis nicht gefunden!"
                        _item.Progress = 0

                    Case ThreadHandlerEnums.UpdateItemEventType.FileNotFound

                        _item.Status = Status.Failure
                        _item.StatusText = "Datei nicht gefunden!"
                        _item.Progress = 0

                    Case ThreadHandlerEnums.UpdateItemEventType.ConnectionError

                        _item.Status = Status.Failure
                        _item.StatusText = "Verbindungsfehler"
                        _item.Progress = 0

                    Case ThreadHandlerEnums.UpdateItemEventType.AuthError

                        _item.Status = Status.Failure
                        _item.StatusText = "Login Falsch"
                        _item.Progress = 0

                    Case ThreadHandlerEnums.UpdateItemEventType.ServerDown

                        _item.Status = Status.Failure
                        _item.StatusText = "Server nicht verfügbar"
                        _item.Progress = 0

                    Case ThreadHandlerEnums.UpdateItemEventType.ServerFull

                        _item.Status = Status.Failure
                        _item.StatusText = "Server voll"
                        _item.Progress = 0

                    Case ThreadHandlerEnums.UpdateItemEventType.InternalServerError

                        _item.Status = Status.Failure
                        _item.StatusText = "Interner Server Fehler"
                        _item.Progress = 0


                    Case ThreadHandlerEnums.UpdateItemEventType.UnknownError

                        _item.Status = Status.Failure
                        _item.StatusText = "Unbekannter Fehler"
                        _item.Progress = 0


                    Case ThreadHandlerEnums.UpdateItemEventType.Retry

                        _item.Status = Status.Retry
                        _item.StatusText = "Retry...Warte..."

                    Case ThreadHandlerEnums.UpdateItemEventType.DownloadRunning

                        _item.Status = Status.Running
                        _item.StatusText = "Downloading..."

                    Case ThreadHandlerEnums.UpdateItemEventType.DownloadComplete

                        _item.Status = Status.Complete
                        _item.StatusText = "Fertig"
                        _item.DownloadSpeed = String.Empty
                        _item.Progress = 100

                    Case ThreadHandlerEnums.UpdateItemEventType.DownloadComplete_HashValid

                        _item.Status = Status.CompleteHashValid
                        _item.StatusText = "Fertig - Hash Überprüfung: OK"
                        _item.DownloadSpeed = String.Empty
                        _item.Progress = 100

                    Case ThreadHandlerEnums.UpdateItemEventType.DownloadComplete_HashInvalid

                        _item.Status = Status.CompleteHashInvalid
                        _item.StatusText = "Fertig - Hash Überprüfung: Fehler"
                        _item.DownloadSpeed = String.Empty
                        _item.Progress = 100

                    Case ThreadHandlerEnums.UpdateItemEventType.FileValdilation

                        _item.Status = Status.FileValdilation
                        _item.StatusText = "Überprüfe Datei..."
                        _item.DownloadSpeed = String.Empty

                    Case ThreadHandlerEnums.UpdateItemEventType.DownloadStopped

                        _item.Status = Status.Stopped
                        _item.StatusText = "Gestoppt"
                        _item.DownloadSpeed = String.Empty


                End Select

                _object.OLV_Package.RefreshItem(_object.OLV_Package.GetItem(_index_item))

                System.Threading.Tasks.Parallel.ForEach(_plugin_connector.Plugins, Sub(_plugin As PluginInterface.PluginInterface.SFDLLoaderPlugin)

                                                                                       If _plugin.SpecialPluginFunctions.Contains(PluginInterface.PluginInterface.SpeicalPluginFunction.PostItemDownload) Then

                                                                                           Dim _thread As New System.Threading.Thread(Sub() _plugin.PostItemDownload(_item, _session_name, _event_type))

                                                                                           _thread.Start()

                                                                                       End If

                                                                                   End Sub)

                If _var_item_speed_size_collection.ContainsKey(_item.FileName) Then
                    Monitor.Enter(_obj_lock_SizeSpeedCollection)
                    _var_item_speed_size_collection.Remove(_item.FileName)
                    Monitor.Exit(_obj_lock_SizeSpeedCollection)
                End If

                Monitor.Enter(_obj_lock_SizeSpeedCollection)
                _var_item_speed_size_collection.Add(_item.FileName, _item)
                Monitor.Exit(_obj_lock_SizeSpeedCollection)

                UpdateSpeedAndRemaingTime()

            End If

        Catch ex As Exception
            Debug.WriteLine("Fehler! Konnte Item nicht updaten!")
        End Try


    End Sub

    Private Sub UpdateItemProgressEvent(_item As OLV_Item, _session_name As String, _download_speed As String, _precent_downloaded As Integer)

        Try

            If Me.InvokeRequired Then
                Me.Invoke(Sub() Me.UpdateItemProgressEvent(_item, _session_name, _download_speed, _precent_downloaded))
            Else

                Dim _index As Integer
                Dim _index_item As Integer
                Dim _object As Ctr_TabPage

                _index = TabControlMain.TabPages.IndexOfKey(_session_name)

                _object = CType(TabControlMain.TabPages.Item(_index).Controls(0), Ctr_TabPage)

                _index_item = _object.OLV_Package.IndexOf(_item)

                _item.Status = Status.Running
                _item.StatusText = "Downloading..."
                _item.Progress = _precent_downloaded
                _item.DownloadSpeed = _download_speed

                _object.OLV_Package.RefreshItem(_object.OLV_Package.GetItem(_index_item))


                If _var_item_speed_size_collection.ContainsKey(_item.FileName) Then
                    Monitor.Enter(_obj_lock_SizeSpeedCollection)
                    _var_item_speed_size_collection.Remove(_item.FileName)
                    Monitor.Exit(_obj_lock_SizeSpeedCollection)
                End If

                Monitor.Enter(_obj_lock_SizeSpeedCollection)
                _var_item_speed_size_collection.Add(_item.FileName, _item)
                Monitor.Exit(_obj_lock_SizeSpeedCollection)

                UpdateSpeedAndRemaingTime()

            End If

        Catch ex As Exception
            Debug.WriteLine("Fehler! Konnte Item nicht updaten!")
        End Try


    End Sub

    Private Sub StopDownloadCompleteEvent()

        Try

            If Me.InvokeRequired Then
                Me.Invoke(Sub() Me.StopDownloadCompleteEvent())
            Else

                tbl_stop_download.Enabled = False
                tbl_start_download.Enabled = True
                _var_download_running = False

                tss_currentspeed.Text = String.Empty
                tss_time_remaining.Text = String.Empty
                tss_progressbar.Value = 0

                tss_status.Text = "Download gestoppt - Bereit"


            End If

        Catch ex As Exception

        End Try

    End Sub

    Private Sub DownloadCompleteEvent(ByVal _connect_error As Boolean, ByVal _session_name As String)

        Try

            If Me.InvokeRequired Then
                Me.Invoke(Sub() Me.DownloadCompleteEvent(_connect_error, _session_name))
            Else

                If _connect_error = True Then
                    Throw New ConnectException
                End If


                If CheckIfSessionsComplete() = True Then


                    RemoveHandler ThreadHandler.PrepareItem, AddressOf PrepareItemEvent
                    RemoveHandler ThreadHandler.UpdateItem, AddressOf UpdateItemEvent
                    RemoveHandler ThreadHandler.UpdateItemProgress, AddressOf UpdateItemProgressEvent
                    RemoveHandler ThreadHandler.PrepareDownloadSFDLSessionProperty, AddressOf PrepareDownloadSFDLSessionPropertyEvent
                    RemoveHandler ThreadHandler.DownloadCompleted, AddressOf DownloadCompleteEvent

                    tbl_stop_download.Enabled = False
                    tbl_start_download.Enabled = True
                    _var_download_running = False

                    tss_currentspeed.Text = String.Empty
                    tss_time_remaining.Text = String.Empty
                    tss_progressbar.Value = 0

                    tss_status.Text = "Download Fertig - Bereit"
                    tss_status.Image = My.Resources.tick


                End If

            End If


        Catch ex As ConnectException

            If Not String.IsNullOrEmpty(_session_name) And _var_sfdl_session_list.ContainsKey(_session_name) Then

                Dim _ctr_tab_page As New Ctr_TabPage

                _ctr_tab_page = GetTabOLVControl(_session_name)

                For Each _item As OLV_Item In _ctr_tab_page.OLV_Package.CheckedObjects
                    UpdateItemEvent(_item, _session_name, ThreadHandlerEnums.UpdateItemEventType.ConnectionError)
                Next

            End If

            DownloadCompleteEvent(False, String.Empty)

        Catch ex As Exception

        End Try

    End Sub


#End Region

#Region "Open SFDL Events und Funktionen"


    Protected Friend Sub OpenSFDLThreadHelper(ByVal _taskid As Integer, Optional ByVal _file As String = Nothing, Optional ByVal _file_list As String() = Nothing, Optional ByVal isSFDLSession As Boolean = False)

        Dim _files2open() As String = Nothing

        If Not _file Is Nothing Then

            Dim _array() As String = {_file}

            _files2open = _array

        End If

        If Not _file_list Is Nothing Then
            _files2open = _file_list
        End If

        If Not _files2open Is Nothing Then

            If isSFDLSession And _files2open.Length > 0 Then

                Dim _open_file_thread As New Thread(Sub() OpenSFDLSessions(_files2open, _taskid))

                AddHandler OpenSFDLSessionComplete, AddressOf OpenSFDLSessionCompleteEvent
                AddHandler AddTabPage, AddressOf AddTabPageEvent
                AddHandler OpenSFDLComplete, AddressOf OpenSFDLCompleteEvent

                _open_file_thread.Start()

            Else

                If Not _files2open.Length = 0 Then

                    Dim _open_file_thread As New Thread(Sub() OpenSFDLFiles(_files2open, _taskid))

                    AddHandler OpenSFDLFileComplete, AddressOf OpenSFDLFileCompleteEvent
                    AddHandler AddTabPage, AddressOf AddTabPageEvent
                    AddHandler OpenSFDLComplete, AddressOf OpenSFDLCompleteEvent
                    AddHandler ThreadHandler.UpdateBulkSession, AddressOf UpdateBulkSessionEvent
                    AddHandler ThreadHandler.BulkSessionComplete, AddressOf BulkSessionCompleteEvent

                    _open_file_thread.Start()

                Else
                    SetTaskComplete(_taskid, "Keine SFDL Dateien geöffnet")
                End If

            End If

        Else
            SetTaskComplete(_taskid, "Keine SFDL Dateien geöffnet")
        End If

    End Sub

    Private Sub AddTabPageEvent(_tab_name As String, _tab_description As String)

        If Me.InvokeRequired Then
            Me.Invoke(Sub() Me.AddTabPageEvent(_tab_name, _tab_description))
        Else
            TabControlMain.TabPages.Add(_tab_name, _tab_description)
        End If

    End Sub

    Private Sub OpenSFDLCompleteEvent(ByVal _taskid As Integer, Optional ByVal _bulk_package As Boolean = False)

        If Me.InvokeRequired Then
            Me.Invoke(Sub() Me.OpenSFDLCompleteEvent(_taskid, _bulk_package))
        Else

            If _bulk_package = False Then


                RemoveHandler OpenSFDLSessionComplete, AddressOf OpenSFDLSessionCompleteEvent
                RemoveHandler OpenSFDLFileComplete, AddressOf OpenSFDLFileCompleteEvent
                RemoveHandler AddTabPage, AddressOf AddTabPageEvent
                RemoveHandler OpenSFDLComplete, AddressOf OpenSFDLCompleteEvent
                RemoveHandler ThreadHandler.UpdateBulkSession, AddressOf UpdateBulkSessionEvent
                RemoveHandler ThreadHandler.BulkSessionComplete, AddressOf BulkSessionCompleteEvent

                SetTaskComplete(_taskid, "SFDL Datei/Session geöffnet")

            Else

                RemoveHandler OpenSFDLSessionComplete, AddressOf OpenSFDLSessionCompleteEvent
                RemoveHandler OpenSFDLFileComplete, AddressOf OpenSFDLFileCompleteEvent
                RemoveHandler AddTabPage, AddressOf AddTabPageEvent
                RemoveHandler OpenSFDLComplete, AddressOf OpenSFDLCompleteEvent

            End If


            For Each _restore_file In IO.Directory.GetFiles(Application.StartupPath, "*.restore", IO.SearchOption.TopDirectoryOnly)
                Try
                    IO.File.Delete(_restore_file)
                Catch ex As Exception
                    Debug.WriteLine("Fehler im Restore File - Datei konnte nicht wiederhergestellt werden")
                End Try
            Next


        End If
    End Sub

    Private Sub OpenSFDLFileCompleteEvent(_olv_file_item_list As List(Of OLV_Item), _sfdl_file_uid As String)

        If Me.InvokeRequired Then
            Me.Invoke(Sub() Me.OpenSFDLFileCompleteEvent(_olv_file_item_list, _sfdl_file_uid))
        Else

            Dim _tab_page_index As Integer = 0
            Dim _ctr_tab_page As New Ctr_TabPage

            _ctr_tab_page.Dock = DockStyle.Fill

            _ctr_tab_page.OLV_Package.AddObjects(_olv_file_item_list)
            _ctr_tab_page.Name = _sfdl_file_uid

            If _settings.MarkAllContainerFiles = True Then
                _ctr_tab_page.OLV_Package.CheckObjects(_ctr_tab_page.OLV_Package.Objects) 'Performance Lag?
            End If

            _ctr_tab_page.OLV_Package.BuildGroups(_ctr_tab_page.olvc_package, SortOrder.Ascending)

            _tab_page_index = TabControlMain.TabPages.IndexOfKey(_sfdl_file_uid)
            TabControlMain.TabPages.Item(_tab_page_index).Controls.Add(_ctr_tab_page)

            End If


    End Sub

    Private Sub OpenSFDLSessionCompleteEvent(_olv_file_item_list As List(Of OLV_Item), _sfdl_session As SFDLSession)

        If Me.InvokeRequired Then
            Me.Invoke(Sub() Me.OpenSFDLSessionCompleteEvent(_olv_file_item_list, _sfdl_session))
        Else

            Dim _tab_page_index As Integer = 0
            Dim _ctr_tab_page As New Ctr_TabPage
            Dim _list_item2check As New List(Of Object)

            If Not _olv_file_item_list.Count = 0 Then

                _ctr_tab_page.Dock = DockStyle.Fill

                _ctr_tab_page.OLV_Package.AddObjects(_olv_file_item_list)
                _ctr_tab_page.Name = _sfdl_session.SFDLSessionName

                _ctr_tab_page.OLV_Package.BuildGroups(_ctr_tab_page.olvc_package, SortOrder.Ascending)

                'Items wieder 'checken'
                For Each _checked_obj As FileInfo In _sfdl_session.CheckedItems
                    Dim _index As Integer = _ctr_tab_page.OLV_Package.FindItemWithText(_checked_obj.FileName).Index
                    _list_item2check.Add(_ctr_tab_page.OLV_Package.GetModelObject(_index))
                Next

                _ctr_tab_page.OLV_Package.CheckObjects(_list_item2check)

                If Not TabControlMain.TabCount = 0 Then
                    _tab_page_index = TabControlMain.TabPages.IndexOfKey(_sfdl_session.SFDLSessionName)
                    TabControlMain.TabPages.Item(_tab_page_index).Controls.Add(_ctr_tab_page)
                End If

            End If

        End If

    End Sub


    Private Sub UpdateBulkSessionEvent(_olv_list As List(Of OLV_Item), _session_name As String, _packagename As String)

        If Me.InvokeRequired Then
            Me.Invoke(Sub() Me.UpdateBulkSessionEvent(_olv_list, _session_name, _packagename))
        Else

            Dim _sfdl_session As New SFDLSession

            _sfdl_session = GetSFDLSessionByKey(_session_name)

            For Each _package In _sfdl_session.SFDLFile.Packages

                If _package.Packagename.ToLower.Equals(_packagename.ToLower) Then

                    _package.BulkFolderMode = False
                    _package.BulkFolderList = Nothing

                    Dim _file_list As New List(Of FileInfo)

                    For Each _olv_item In _olv_list

                        Dim _item As New SFDL.File.FileInfo

                        With _item

                            .DirectoryPath = _olv_item.DirectoryPath
                            .DirectoryRoot = _olv_item.DirectoryRoot
                            .FileFullPath = _olv_item.FileFullPath
                            .FileName = _olv_item.FileName
                            .FileSize = _olv_item.FileSize
                            .FileHash = _olv_item.FileHash
                            .FileHashType = _olv_item.FileHashType
                            .PackageName = _olv_item.PackageName

                        End With

                        _file_list.Add(_item)

                    Next

                    Dim _olv_control As Ctr_TabPage

                    _olv_control = GetTabOLVControl(_session_name)

                    _olv_control.OLV_Package.AddObjects(_olv_list)

                    _olv_control.OLV_Package.BuildGroups(_olv_control.olvc_package, SortOrder.Ascending)

                    If _package.FileList.Count = 0 Then
                        _package.FileList = _file_list
                    Else
                        _package.FileList.AddRange(_file_list)
                    End If

                    UpdateSFDLSession(New KeyValuePair(Of String, SFDLSession)(_sfdl_session.SFDLSessionName, _sfdl_session))

                End If

            Next


        End If

    End Sub

    Private Sub BulkSessionCompleteEvent(ByVal _error As Boolean, ByVal _taskid As Integer)

        If Me.InvokeRequired Then
            Me.Invoke(Sub() Me.BulkSessionCompleteEvent(_error, _taskid))
        Else


            If _error = True Then
                SetTaskComplete(_taskid, "Fehler beim ermitteln der Dateien", True)
            End If

            SetTaskComplete(_taskid, "SFDL erfolgreich geöffnet")

            RemoveHandler OpenSFDLSessionComplete, AddressOf OpenSFDLSessionCompleteEvent
            RemoveHandler OpenSFDLFileComplete, AddressOf OpenSFDLFileCompleteEvent
            RemoveHandler AddTabPage, AddressOf AddTabPageEvent
            RemoveHandler OpenSFDLComplete, AddressOf OpenSFDLCompleteEvent
            RemoveHandler ThreadHandler.UpdateBulkSession, AddressOf UpdateBulkSessionEvent
            RemoveHandler ThreadHandler.BulkSessionComplete, AddressOf BulkSessionCompleteEvent


        End If


    End Sub

#End Region

#Region "Protected Friend Sub's"

    Protected Friend Sub AddSFDLSession(ByVal _session As KeyValuePair(Of String, SFDLSession))

        Monitor.Enter(_obj_lock_SFDLSessionList)

        _var_sfdl_session_list.Add(_session.Key, _session.Value)

        Monitor.Exit(_obj_lock_SFDLSessionList)

    End Sub

    Protected Friend Sub UpdateSFDLSession(ByVal _session As KeyValuePair(Of String, SFDLSession))

        If _var_sfdl_session_list.ContainsKey(_session.Key) Then

            Monitor.Enter(_obj_lock_SFDLSessionList)

            _var_sfdl_session_list.Remove(_session.Key)
            _var_sfdl_session_list.Add(_session.Key, _session.Value)

            Monitor.Exit(_obj_lock_SFDLSessionList)

        End If

    End Sub

    Protected Friend Sub SetSFDLSessionState(ByVal _session_name As String, ByVal _state As SFDLSessionEnums.SFDLSessionState)

        If _var_sfdl_session_list.ContainsKey(_session_name) Then

            Monitor.Enter(_obj_lock_SFDLSessionList)

            Select Case _state

                Case SFDLSessionEnums.SFDLSessionState.DonwloadStopped

                    _var_sfdl_session_list.Item(_session_name).SessionState = SFDLSessionEnums.SFDLSessionState.DonwloadStopped

                Case SFDLSessionEnums.SFDLSessionState.DownloadComplete

                    _var_sfdl_session_list.Item(_session_name).SessionState = SFDLSessionEnums.SFDLSessionState.DownloadComplete


                Case SFDLSessionEnums.SFDLSessionState.DownloadRunning

                    _var_sfdl_session_list.Item(_session_name).SessionState = SFDLSessionEnums.SFDLSessionState.DownloadRunning

                Case SFDLSessionEnums.SFDLSessionState.Queued

                    _var_sfdl_session_list.Item(_session_name).SessionState = SFDLSessionEnums.SFDLSessionState.Queued


            End Select

            Monitor.Exit(_obj_lock_SFDLSessionList)

        End If

    End Sub

    Protected Friend Sub RemoveSFDLSessionByKey(ByVal _session_key As String)

        If _var_sfdl_session_list.ContainsKey(_session_key) Then

            Monitor.Enter(_obj_lock_SFDLSessionList)

            _var_sfdl_session_list.Remove(_session_key)

            Monitor.Exit(_obj_lock_SFDLSessionList)
        End If

    End Sub

    Protected Friend Sub AddItem2SizeSpeedCollection(ByVal _olv_item As OLV_Item)

        Monitor.Enter(_obj_lock_SizeSpeedCollection)
        _var_item_speed_size_collection.Add(_olv_item.FileName, _olv_item)
        Monitor.Exit(_obj_lock_SizeSpeedCollection)

    End Sub

    Protected Friend Sub RemoveItemFromSizeSpeedCollection(ByVal _olv_item As OLV_Item)


        If _var_item_speed_size_collection.ContainsKey(_olv_item.FileName) Then

            Monitor.Enter(_obj_lock_SizeSpeedCollection)

            _var_item_speed_size_collection.Remove(_olv_item.FileName)

            Monitor.Exit(_obj_lock_SizeSpeedCollection)

        End If

    End Sub


    Protected Friend Sub UpdateSpeedAndRemaingTime()

        If Me.InvokeRequired Then
            Me.Invoke(Sub() Me.UpdateSpeedAndRemaingTime())
        Else

            Dim _total_speed As Double = 0
            Dim _total_size As Double = 0
            Dim _total_size_downloaded As Double = 0
            Dim _size_remaining As Double = 0
            Dim _time_remaining As Double = 0

            Dim _tools As New Tools

            If _var_download_stop_in_progress = False Then

                Try

                    For Each _item In _var_item_speed_size_collection

                        Dim _raw_speed As String = String.Empty

                        If _item.Value.Status = Status.Running Or _item.Value.Status = Status.Waiting Then

                            If Not String.IsNullOrEmpty(_item.Value.DownloadSpeed) Then
                                _raw_speed = _item.Value.DownloadSpeed.ToString.Replace("KB/s", "").Trim
                                _total_speed += Double.Parse(_raw_speed)
                            End If

                            _total_size += _item.Value.FileSize
                            _total_size_downloaded += _item.Value.SizeDownloaded

                        End If

                    Next

                    _size_remaining = _total_size - _total_size_downloaded

                    _time_remaining = Math.Round(Double.Parse(CStr(((_size_remaining / 1024) / _total_speed) / 60)), 2)

                    _var_remaining_time = String.Format("Verbleibende Zeit: {0}", _tools.ConvertDecimal2Time(_time_remaining).ToString)
                    _var_download_speed = String.Format("Speed: {0} KB/s", _total_speed)
                    _var_download_progress = CInt((_total_size_downloaded / _total_size) * 100).ToString & "%"

                    tss_time_remaining.Text = _var_remaining_time
                    tss_currentspeed.Text = _var_download_speed
                    tss_progressbar.Value = CInt((_total_size_downloaded / _total_size) * 100)

                Catch ex As Exception

                End Try

            End If

        End If

    End Sub

#End Region

#Region "Public Subs"

    Public Function DownloadFilePath(ByVal _settings As Settings, ByVal _sfdl_session As SFDLSession, ByVal _olv_item As OLV_Item) As String
        Return GetDownloadFilePath(_settings, _sfdl_session, _olv_item)
    End Function

#End Region

#Region "Private Sub's"

    Private Function CheckIfSessionsComplete() As Boolean

        Dim _download_complete As Boolean = False

        Try

            _logger.WriteLog("CheckIfSessionsComplete", "Prüfe ob Sessions fertig sind...")

            For Each _tab As TabPage In TabControlMain.TabPages

                Dim _olv_control As Ctr_TabPage
                Dim _session_complete As Boolean = False
                Dim _session_downloaded_file As Boolean = False

                Try

                    _logger.WriteLog("CheckIfSessionsComplete", "Prüfe Session " & _tab.Name)

                    If GetSFDLSessionByKey(_tab.Name).SessionState = SFDLSessionEnums.SFDLSessionState.DownloadComplete Then
                        Throw New DownloadCompletedException
                    End If

                    _olv_control = GetTabOLVControl(_tab.Name)

                    For Each _item As OLV_Item In _olv_control.OLV_Package.CheckedObjects

                        _logger.WriteLog("CheckIfSessionsComplete", "Item Status:" & _item.Status.ToString)

                        Select Case _item.Status

                            Case Status.Failure

                                _session_complete = True

                            Case Status.CompleteHashInvalid

                                _session_complete = True

                            Case Status.FileValdilation

                                _session_complete = False

                            Case Status.Queued

                                _session_complete = False

                            Case Status.Retry

                                _session_complete = False

                            Case Status.Running

                                _session_complete = False

                            Case Status.Stopped

                                _session_complete = True

                            Case Status.Waiting

                                _session_complete = False

                            Case Status.Complete

                                _session_complete = True
                                _session_downloaded_file = True

                            Case Status.CompleteHashValid

                                _session_complete = True
                                _session_downloaded_file = True

                        End Select

                    Next

                    If _session_complete = True Then

                        Dim _sfdl_session As New SFDLSession

                        _sfdl_session = GetSFDLSessionByKey(_tab.Name)
                        _sfdl_session.SessionState = SFDLSessionEnums.SFDLSessionState.DownloadComplete

                        _sfdl_session.TimeDownloadStopped = Now

                        Debug.WriteLine(DateDiff(DateInterval.Second, Now, _sfdl_session.TimeDownloadStopped))

                        UpdateSFDLSession(New KeyValuePair(Of String, SFDLSession)(_sfdl_session.SFDLSessionName, _sfdl_session))

                        System.Threading.Tasks.Parallel.ForEach(_plugin_connector.Plugins, Sub(_plugin As SFDL.PluginInterface.PluginInterface.SFDLLoaderPlugin)


                                                                                               If _plugin.SpecialPluginFunctions.Contains(PluginInterface.PluginInterface.SpeicalPluginFunction.PostSessionDownload) Then

                                                                                                   Dim _thread As New System.Threading.Thread(Sub() _plugin.PostSessionDownload(_sfdl_session, _session_downloaded_file))

                                                                                                   _thread.Start()

                                                                                               End If

                                                                                           End Sub)

                        _logger.WriteLog("CheckIfSessionsComplete", "Session " & _sfdl_session.SFDLSessionName & " ist fertig heruntergeladen!")

                        _download_complete = True

                    Else

                        If _olv_control.OLV_Package.CheckedObjects.Count = 0 Then
                            SetSFDLSessionState(_tab.Name, SFDLSessionEnums.SFDLSessionState.DownloadComplete)
                        Else
                            SetSFDLSessionState(_tab.Name, SFDLSessionEnums.SFDLSessionState.DownloadRunning)
                        End If

                        _logger.WriteLog("CheckIfSessionsComplete", "Session " & _tab.Name & " ist NICHT fertig heruntergeladen!")
                        _download_complete = False

                    End If

                Catch ex As DownloadCompletedException
                    _logger.WriteLog("CheckIfSessionsComplete", "Session muss nicht geprüft werden da Sie schon fertig ist!")
                    _download_complete = True
                End Try

            Next

        Catch ex As DownloadCompletedException


        Catch ex As Exception
            _logger.WriteLog("CheckIfSessionsComplete", ex.Message, LogType.Fehler)
        Finally
            _logger.WriteLog("CheckIfSessionsComplete", "Prüfen der Sessions abgeschlossen!")
        End Try

        Return _download_complete

    End Function

#End Region

#Region "Main Form Events"

    Private Sub frm_main_Load(sender As Object, e As EventArgs) Handles MyBase.Load

        Try

            _settings = New Settings
            _var_sfdl_session_list = New Dictionary(Of String, SFDLSession)

            'Plugins laden 

            'Verzeichnisse erstellen
            If IO.Directory.Exists(CONST_PLUGINDIR) = False Then
                IO.Directory.CreateDirectory(CONST_PLUGINDIR)
                _logger.WriteLog("Form Main - Load", "Plugin Verzeichniss erstellt!")
            End If


            If _tools.IsLinux = False Then 'Wenn es sich um ein Windows System handelt

                CONST_SETTINGSFILEPATH = IO.Path.Combine(Environment.GetEnvironmentVariable("appdata"), "SFDL.NET 2\settings.xml")

                _logger.WriteLog("Form Main - Load", "Windows System erkannt!")

                If IO.Directory.Exists(IO.Path.GetDirectoryName(CONST_SETTINGSFILEPATH)) = False Then
                    IO.Directory.CreateDirectory(IO.Path.GetDirectoryName(CONST_SETTINGSFILEPATH))
                End If

                If IO.File.Exists(CONST_SETTINGSFILEPATH) Then

                    _settings = CType(_tools.XMLDeSerialize(_settings, CONST_SETTINGSFILEPATH), Settings)

                    _logger.WriteLog("Form Main - Load", "Einstellungen geladen!")

                Else

                    'Default Werte laden

                    _logger.WriteLog("Form Main - Load", "Keine Einstellungen gefunden - Lade Default Einstellungen")

                    _settings.DownloadDirectory = IO.Path.Combine(Application.StartupPath, "Download")

                    If IO.Directory.Exists(IO.Path.Combine(Application.StartupPath, "Download")) = False Then
                        IO.Directory.CreateDirectory(IO.Path.Combine(Application.StartupPath, "Download"))
                    End If

                    _settings.ExistingFileHandling = ExistingFileHandling.ResumeFile
                    _settings.UpdateCheck = True
                    _settings.ClicknLoadEnabled = True
                    _settings.Send2Tray = True
                    _settings.MaxDownloadThreads = 3
                    _settings.MaxRetry = 3
                    _settings.MarkAllContainerFiles = True

                    'Danach dann die Default Einstellungen gleich speichern

                    _tools.XMLSerialize(_settings, CONST_SETTINGSFILEPATH)

                    _logger.WriteLog("Form Main - Load", "Default Einstellungen geladen!")

                End If

                Try 'Versuchen die Dateiendung .sfdl und den URI Handler zu registrieren 

                    Dim _file_assoiciation As New FileAssociation

                    _file_assoiciation.Extension = "sfdl"
                    _file_assoiciation.ContentType = "application/sfdl.net"
                    _file_assoiciation.FullName = "SFDL.NET Files"
                    _file_assoiciation.IconIndex = 0
                    _file_assoiciation.IconPath = IO.Path.Combine(Application.StartupPath, "Icon.ico")
                    _file_assoiciation.ProperName = "SFDL.NET File"
                    _file_assoiciation.AddCommand("open", Application.ExecutablePath.ToString & " " & Chr(34) & "%1" & Chr(34))

                    _file_assoiciation.Create()

                    _logger.WriteLog("Form Main - Load", "SFDL Extension registriert!")

                    If _settings.ClicknLoadEnabled Then

                        Dim _uri_handler_association As New URIHandlerAssociation

                        _uri_handler_association.RegisterSFDLURIHandler(Application.ExecutablePath)

                        _logger.WriteLog("Form Main - Load", "SFDL URI Handler registriert!")

                    End If

                Catch ex As Exception
                    _logger.WriteLog("Form Main - Load", ex.Message, LogType.Fehler)
                End Try

                If _settings.PreventStandby Then

                    Dim _standby_handler As New StandyHandler

                    _standby_handler.PreventStandby()

                    _logger.WriteLog("Form Main - Load", "Standby wird nun verhindert!")

                End If

            Else 'Wenn es sich um ein Linux System handelt

                CONST_SETTINGSFILEPATH = IO.Path.Combine(Application.StartupPath, "config\settings.xml")

                _logger.WriteLog("Form Main", "Linux System erkannt!")

                If IO.Directory.Exists(IO.Path.GetDirectoryName(CONST_SETTINGSFILEPATH)) = False Then
                    IO.Directory.CreateDirectory(IO.Path.GetDirectoryName(CONST_SETTINGSFILEPATH))
                End If

                If IO.File.Exists(CONST_SETTINGSFILEPATH) Then

                    _settings = CType(_tools.XMLDeSerialize(_settings, CONST_SETTINGSFILEPATH), Settings)

                    _logger.WriteLog("Form Main - Load", "Einstellungen geladen!")

                Else

                    _logger.WriteLog("Form Main - Load", "Keine Einstellungen gefunden - Lade Default Einstellungen")

                    _settings.ExistingFileHandling = ExistingFileHandling.ResumeFile
                    _settings.MaxDownloadThreads = 3
                    _settings.MaxRetry = 3
                    _settings.UpdateCheck = False
                    _settings.ClicknLoadEnabled = False
                    _settings.Send2Tray = False
                    _settings.MarkAllContainerFiles = True

                    'Danach dann die Default Einstellungen gleich speichern

                    _tools.XMLSerialize(_settings, CONST_SETTINGSFILEPATH)

                    _logger.WriteLog("Form Main - Load", "Default Einstellungen geladen!")

                End If

            End If


            'Plugins laden
            _logger.WriteLog("Form Main - Load", "Lade Plugins...")
            _plugin_connector.LoadPlugins(CONST_PLUGINDIR)
            _logger.WriteLog("Form Main - Load", "Plugins geladen!")

            System.Threading.Tasks.Parallel.ForEach(_plugin_connector.Plugins, (Sub(_plugin As SFDL.PluginInterface.PluginInterface.SFDLLoaderPlugin) _plugin.Load()))

            _logger.WriteLog("Form Main - Load", _plugin_connector.Plugins.Length & " Plugins geladen!")

            _logger.WriteLog("Form Main - Load", "Verarbeite CommandButton Plugins...")
            ProgressCommandButtonPlugins()
            _logger.WriteLog("Form Main - Load", "OK - CommandButton PLugins verarbeitet!")

            If My.Application.CommandLineArgs.Count >= 1 Then 'Wenn Parameter vorhanden

                Dim _file As String = String.Empty

                _logger.WriteLog("Form Main - Load", "Parse und verabeite Start Parameter...")

                If My.Application.CommandLineArgs(0).StartsWith("sfdl://") Then

                    _file = ProgressClicknLoadPlugins(My.Application.CommandLineArgs(0))

                    If String.IsNullOrEmpty(_file) Then
                        Throw New PluginNotFoundException
                    End If

                    If IO.File.Exists(_file) Then
                        OpenSFDLThreadHelper(AddTask("Öffne SFDL Datei(en)"), _file)
                    End If

                Else

                    If IO.File.Exists(_file) Then
                        OpenSFDLThreadHelper(AddTask("Öffne SFDL Datei(en)"), _file)
                    End If

                End If

            End If

            _logger.WriteLog("Form Main - Load", "Stelle SFDL Sessions wieder her....")

            OpenSFDLThreadHelper(AddTask("Öffne SFDL Sessions.."), Nothing, IO.Directory.GetFiles(Application.StartupPath, "*.restore", IO.SearchOption.TopDirectoryOnly), True)

            If Not TabControlMain.TabCount = 0 Then
                RefreshSFDLInfoIcon(TabControlMain.SelectedTab.Name)
            End If

        Catch ex As UserCancelException

            'Benutzer hat abgebrochen

        Catch ex As PluginNotFoundException

            MessageBox.Show("ClicknLoad ist fehlgeschlagen da leider kein passendes Plugin gefunden wurde welchen den Hoster unterstützt!", "ClicknLoad Fehler", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

        Catch ex As Exception

            MessageBox.Show("Fehler" & vbNewLine & "Fehlermeldung: " & ex.Message, "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
            _logger.WriteLog("Form Main", ex.Message, LogType.Fehler)

        End Try

    End Sub

    Private Sub frm_main_FormClosing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing

        'Aktuell geöffnete SFDL File wegschreiben damit der Zustand beim nächsten Start wiederhergestellt werden kann
        Try

            If _var_download_running = True Then

                If MessageBox.Show("Es läuft noch ein Download! - Willst du trotzdem die Anwendung beenden?", "Achtung - Download läuft!", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = Windows.Forms.DialogResult.No Then
                    e.Cancel = True
                End If

            End If

            _logger.WriteLog("Form Main - Closing", "Speichere geöffnete SFDL Sessions..")

            For Each _session In _var_sfdl_session_list

                Dim _tab_page As TabPage
                Dim _olv_control As Ctr_TabPage
                Dim _list_checked As New List(Of OLV_Item)

                _tab_page = TabControlMain.TabPages.Item(TabControlMain.TabPages.IndexOfKey(_session.Key.ToString))

                _olv_control = CType(_tab_page.Controls(0), Ctr_TabPage)

                For Each _checked_item In _olv_control.OLV_Package.CheckedObjects
                    _list_checked.Add(CType(_checked_item, OLV_Item))
                Next

                _session.Value.CheckedItems = _list_checked

                _tools.XMLSerialize(_session.Value, IO.Path.Combine(Application.StartupPath, IO.Path.GetRandomFileName & ".restore"))

            Next

            _logger.WriteLog("Form Main - Closing", "SFDL Sessions gespeichert!")

            _logger.WriteLog("Form Main - Closing", "Verstecke Restore Files...")

            For Each _file In IO.Directory.GetFiles(Application.StartupPath, "*.restore", IO.SearchOption.TopDirectoryOnly)

                Dim _file_info As New IO.FileInfo(_file)
                _file_info.Attributes = IO.FileAttributes.Hidden

            Next

            _logger.WriteLog("Form Main - Closing", "Abgeschlossen")


        Catch ex As Exception
            _logger.WriteLog("Form Main - Closing", ex.Message, LogType.Fehler)
        End Try

    End Sub


#End Region

#Region "Main Form Button Events & Co."


    Private Sub tbl_start_download_Click(sender As Object, e As EventArgs) Handles tbl_start_download.Click

        Dim _thread_handler As New ThreadHandler
        Dim _thread As System.Threading.Thread

        Try

            If _var_sfdl_session_list.Count = 0 Then
                Throw New Exception("Keine SFDL geladen!")
            End If

            _logger.WriteLog("Form Main - Start Download", "Bereite GUI für Startenden Download vor...")

            'Lock Controls
            tbl_start_download.Enabled = False
            tbl_stop_download.Enabled = True

            _var_download_running = True
            _var_download_stop_in_progress = False

            _var_item_speed_size_collection.Clear()
            _obj_lock_SizeSpeedCollection = New Object

            tss_status.Text = "Downloading..."
            tss_status.Image = My.Resources.download

            For Each _tab As TabPage In TabControlMain.TabPages

                Dim _olv_control As Ctr_TabPage

                _olv_control = GetTabOLVControl(_tab.Name)

                For Each _item In _olv_control.OLV_Package.Objects

                    Dim _olv_item As OLV_Item

                    _olv_item = TryCast(_item, OLV_Item)

                    If _olv_item.Checked Then
                        _olv_item.Status = Status.Waiting
                        _olv_item.StatusText = "Warte..."
                        _var_item_speed_size_collection.Add(_olv_item.FileName, _olv_item)
                    Else
                        _olv_item.Status = Status.Queued
                        _olv_item.StatusText = String.Empty
                    End If

                    _olv_item.DownloadSpeed = String.Empty
                    _olv_item.Progress = 0

                    _olv_control.OLV_Package.RefreshObject(_olv_item)

                Next

            Next

            'Jede Session wieder auf den STatus Queued setzten
            For Each _session In _var_sfdl_session_list
                _session.Value.SessionState = SFDLSessionEnums.SFDLSessionState.Queued
            Next

            _logger.WriteLog("Form Main - Start Download", "OK - GUI vorbereitet")

            _logger.WriteLog("Form Main - Start Download", "Starte Download ThreadHandler....")

            _thread = New System.Threading.Thread(Sub() _thread_handler.HandleThreads())

            AddHandler ThreadHandler.PrepareItem, AddressOf PrepareItemEvent 'Event + Invoke wird benötigt um auf GUI zugreifen zu können
            AddHandler ThreadHandler.UpdateItem, AddressOf UpdateItemEvent
            AddHandler ThreadHandler.UpdateItemProgress, AddressOf UpdateItemProgressEvent
            AddHandler ThreadHandler.PrepareDownloadSFDLSessionProperty, AddressOf PrepareDownloadSFDLSessionPropertyEvent
            AddHandler ThreadHandler.DownloadCompleted, AddressOf DownloadCompleteEvent

            _thread.Start()

            _logger.WriteLog("Form Main - Start Download", "OK - Thread Handler gestartet")

        Catch ex As Exception

            _logger.WriteLog("Form Main - Start Download", ex.Message, LogType.Fehler)

            MessageBox.Show("Fehler beim starten des Downloads!" & vbNewLine & "Fehlermeldung: " & ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning)

            tbl_start_download.Enabled = True
            tbl_stop_download.Enabled = False
            tss_status.Text = "Fehler beim Start"

        End Try

    End Sub

    Private Sub tbl_stop_download_Click(sender As Object, e As EventArgs) Handles tbl_stop_download.Click

        Dim _thread_handler As New ThreadHandler
        Dim _thread As System.Threading.Thread

        Try

            If MessageBox.Show("Willst du den Download wirklich stoppen?", "Download stoppen?", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = Windows.Forms.DialogResult.Yes Then

                _logger.WriteLog("Form Main - Stop Download", "Starte 'Stop Download' Thread....")

                _var_download_stop_in_progress = True

                _thread = New System.Threading.Thread(Sub() _thread_handler.StopThreads())

                AddHandler ThreadHandler.StopDownloadComplete, AddressOf StopDownloadCompleteEvent

                _thread.Start()

                _logger.WriteLog("Form Main - Stop Download", "OK - Thread gestartet")

            End If

        Catch ex As Exception
            _logger.WriteLog("Form Main - Stop Download", ex.Message, LogType.Fehler)
            MessageBox.Show("Downloads konnten nicht gestoppt werden!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

    End Sub

    Private Sub ÖffnenToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles ÖffnenToolStripMenuItem.Click

        Try

            If Not OFD_SFDL.ShowDialog = Windows.Forms.DialogResult.Cancel Then

                If Not TabControlMain.TabCount = 0 Then 'Fragen ob evtl. bereits geöffnete Dateien verworfen werden sollen

                    If MessageBox.Show("Es ist bereits mind. eine SFDL Datei geöffnet." & vbNewLine & "Möchtest du das die SFDL Datei hinzugefügt wird?", "Hinzufügen oder ersetzten", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = Windows.Forms.DialogResult.No Then
                        ClearTabPages()
                    End If

                End If

                'tss_status_top.Text = "Öffne SFDL Datei(en)...."
                'tss_progressbar_top.Style = ProgressBarStyle.Marquee

                OpenSFDLThreadHelper(AddTask("Öffne SFDL Datei(en)"), Nothing, OFD_SFDL.FileNames)

            End If

            If Not TabControlMain.TabCount = 0 Then
                RefreshSFDLInfoIcon(TabControlMain.SelectedTab.Name)
            End If

        Catch ex As UserCancelException
            'Benutzer hat abgebrochen

        Catch ex As Exception
            MessageBox.Show("Fehler beim Öffnen der SFDL Dateie(n)" & vbNewLine & "Fehlermeldung: " & ex.Message, "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

    End Sub

    Private Sub EinstellungenToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles EinstellungenToolStripMenuItem.Click

        Try

            AddHandler frm_settings.SettingsUpdated, AddressOf frm_setting_settings_update_event

            frm_settings.ShowDialog()
            ' frm_settings.Dispose()

            RemoveHandler frm_settings.SettingsUpdated, AddressOf frm_setting_settings_update_event

        Catch ex As Exception
            MessageBox.Show("Fehler beim Anwenden der Einstellungen!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

    End Sub

    Private Sub BeendenToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles BeendenToolStripMenuItem.Click
        Application.Exit()
    End Sub

    Private Sub AlleFertigenDownloadsEntfernenToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles AlleFertigenDownloadsEntfernenToolStripMenuItem.Click
        If MessageBox.Show("Wirklich alle fertigen Downloads entfernen?", "Sicher?", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = Windows.Forms.DialogResult.Yes Then
            ClearCompletedDownloadTabs()
        End If

    End Sub

    Private Sub AlleDownloadsEntfernenToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles AlleDownloadsEntfernenToolStripMenuItem.Click

        If MessageBox.Show("Wirklick alle Downloads entfernen?", "Sicher?", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = Windows.Forms.DialogResult.Yes Then
            ClearTabPages()
        End If

    End Sub


    Private Sub PluginManagerToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles PluginManagerToolStripMenuItem.Click
        frm_pluginmanager.ShowDialog()
        frm_pluginmanager.Dispose()
    End Sub


    Private Sub AllesMakierenToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles AllesMakierenToolStripMenuItem.Click
        If Not TabControlMain.TabCount = 0 Then
            GetTabOLVControl(TabControlMain.SelectedTab.Name).OLV_Package.CheckObjects(GetTabOLVControl(TabControlMain.SelectedTab.Name).OLV_Package.Objects)
        End If
    End Sub

    Private Sub AllesDeToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles AllesDeToolStripMenuItem.Click
        If Not TabControlMain.TabCount = 0 Then
            GetTabOLVControl(TabControlMain.SelectedTab.Name).OLV_Package.UncheckObjects(GetTabOLVControl(TabControlMain.SelectedTab.Name).OLV_Package.Objects)
        End If
    End Sub

    Private Sub TabControlMain_SelectedIndexChanged(sender As Object, e As EventArgs) Handles TabControlMain.SelectedIndexChanged
        If Not TabControlMain.TabCount = 0 Then
            RefreshSFDLInfoIcon(TabControlMain.SelectedTab.Name)
        Else
            tsdbb_server_information.Visible = False
        End If
    End Sub

#End Region

#Region "Tests"

    'Private Sub TestToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles TestToolStripMenuItem.Click

    '    Dim _sfdl_file As New SFDLFile
    '    Dim _sfdl_connection_info As New ConnectionInfo

    '    With _sfdl_connection_info

    '        .AuthRequired = False
    '        .CharacterEncoding = CharacterEncoding.CharacterEncoding.Standard
    '        .DataConnectionType = Net.FtpClient.FtpDataConnectionType.AutoPassive
    '        .DataType = Net.FtpClient.FtpDataType.Binary
    '        .EncryptionMode = Net.FtpClient.FtpEncryptionMode.Explicit
    '        .Host = "ftp.serv-u.com"
    '        .Port = 21
    '        .UseSpecialCharacterFix = False
    '        .Username = "leech"
    '        .Password = "leech"

    '    End With

    '    _sfdl_file.ConnectionInfo = _sfdl_connection_info
    '    _sfdl_file.Description = "Serv-U FTPS Test"
    '    _sfdl_file.Encrypted = False
    '    _sfdl_file.MaxDownloadThreads = 1
    '    _sfdl_file.SFDLFileVersion = 6
    '    _sfdl_file.Uploader = "Seji"


    '    Dim _package As New SFDLPackage
    '    Dim _bulk_folder As New BulkFolder
    '    Dim Bulk_list As New List(Of BulkFolder)

    '    _package.BulkFolderMode = True
    '    _package.Packagename = "Serv-U FTPS Test"

    '    _bulk_folder.BulkFolderName = "/"
    '    _bulk_folder.BulkFolderPath = "/"
    '    _bulk_folder.PackageName = "Serv-U FTPS Test"

    '    Bulk_list.Add(_bulk_folder)

    '    _package.BulkFolderList = Bulk_list

    '    _sfdl_file.Packages.Add(_package)


    '    _tools.XMLSerialize(_sfdl_file, "C:\Users\hofmann\Documents\ftps_test.sfdl")

    'End Sub

    'Private Sub TestToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles TestToolStripMenuItem.Click

    '    Dim _sfdl_file As New SFDLFile
    '    Dim _sfdl_connection_info As New ConnectionInfo
    '    Dim _file_list As New List(Of FileInfo)
    '    Dim _file_list2 As New List(Of FileInfo)

    '    Dim _package As New SFDLPackage
    '    Dim _package2 As New SFDLPackage

    '    _sfdl_file.ConnectionInfo = _sfdl_connection_info
    '    _sfdl_file.Description = "Test Download"
    '    _sfdl_file.Encrypted = False
    '    _sfdl_file.MaxDownloadThreads = 3
    '    _sfdl_file.SFDLFileVersion = 6
    '    _sfdl_file.Uploader = "Seji"


    '    _package.Packagename = "Package 1"

    '    For x = 0 To 9

    '        Dim _file As New FileInfo

    '        _file.DirectoryPath = "\files\movieac3\cd1\"
    '        _file.DirectoryRoot = "\files\movieac3\"
    '        _file.FileFullPath = String.Format("\files\movieac3\cd1\file0{0}.rar", x)
    '        _file.FileName = String.Format("file0{0}.rar", x)
    '        _file.FileSize = 100000
    '        _file.MD5Hash = String.Empty
    '        _file.PackageName = _package.Packagename

    '        _file_list.Add(_file)

    '    Next

    '    _package.FileList = _file_list



    '    _package2.Packagename = "package 2"

    '    For x = 0 To 9

    '        Dim _file As New FileInfo

    '        _file.DirectoryPath = "\files\movieac3\cd1\"
    '        _file.DirectoryRoot = "\files\movieac3\"
    '        _file.FileFullPath = String.Format("\files\movieac3\cd1\file0{0}.rar", x)
    '        _file.FileName = String.Format("file0{0}.rar", x)
    '        _file.FileSize = 100000
    '        _file.MD5Hash = String.Empty
    '        _file.PackageName = _package2.Packagename

    '        _file_list2.Add(_file)

    '    Next

    '    _package2.FileList = _file_list2

    '    _sfdl_file.Packages.Add(_package)
    '    _sfdl_file.Packages.Add(_package2)


    '    Debug.WriteLine("Test SFDL erstellt")

    '    _tools.XMLSerialize(_sfdl_file, "test.sfdl")

    '    Debug.WriteLine("SFDL geschrieben!")


    'End Sub

#End Region

#Region "TabControl Control Event's"


    Private Sub SchließenToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles SchließenToolStripMenuItem.Click

        If MessageBox.Show("Datei wirklich schließen?", "SFDL Datei schließen", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = Windows.Forms.DialogResult.Yes Then

            RemoveSFDLSessionByKey(TabControlMain.SelectedTab.Name)

            TabControlMain.TabPages.Remove(TabControlMain.SelectedTab)

        End If

    End Sub


    Private Sub TabControlMain_DragEnter(sender As Object, e As DragEventArgs) Handles TabControlMain.DragEnter

        If e.Data.GetDataPresent(DataFormats.FileDrop) Then
            e.Effect = DragDropEffects.Copy
        End If

    End Sub

    Private Sub TabControlMain_DragDrop(sender As Object, e As DragEventArgs) Handles TabControlMain.DragDrop

        Dim _sfdl_file_path As String = String.Empty
        Dim _files() As String

        Try

            _files = CType(e.Data.GetData(DataFormats.FileDrop), String())

            If Not _files Is Nothing Then

                If Not _files.Length = 0 Then
                    _sfdl_file_path = _files(0).ToString
                End If

                If IO.Path.GetExtension(_sfdl_file_path).ToLower = ".sfdl" Then

                    'tss_status_top.Text = "Öffne SFDL Datei(en)...."
                    'tss_progressbar_top.Style = ProgressBarStyle.Marquee

                    OpenSFDLThreadHelper(AddTask("Öffne SFDL Datei(en)"), _sfdl_file_path)

                End If

            End If

        Catch ex As UserCancelException

        Catch ex As Exception
            MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

    End Sub

    'Tab SwitchFunktionen von http://dotnetrix.co.uk/tabcontrol.htm#tip7

    Private Sub TabControlMain_MouseMove(sender As Object, e As MouseEventArgs) Handles TabControlMain.MouseMove
        If e.Button <> MouseButtons.Left Then Return

        Dim r As Rectangle = New Rectangle(DragStartPosition, Size.Empty)
        r.Inflate(SystemInformation.DragSize)

        Dim tp As TabPage = HoverTab()

        If Not tp Is Nothing Then
            If Not r.Contains(e.X, e.Y) Then
                TabControlMain.DoDragDrop(tp, DragDropEffects.All)
            End If
        End If

        DragStartPosition = Point.Empty
    End Sub

    Private Sub TabControlMain_MouseDown(sender As Object, e As MouseEventArgs) Handles TabControlMain.MouseDown
        DragStartPosition = New Point(e.X, e.Y)
    End Sub

    Private Sub TabControlMain_DragOver(sender As Object, e As DragEventArgs) Handles TabControlMain.DragOver

        Dim hover_Tab As TabPage = HoverTab()
        If hover_Tab Is Nothing Then
            e.Effect = DragDropEffects.None
        Else
            If e.Data.GetDataPresent(GetType(TabPage)) Then

                e.Effect = DragDropEffects.Move
                Dim drag_tab As TabPage = DirectCast(e.Data.GetData(GetType(TabPage)), TabPage)

                If hover_Tab Is drag_tab Then Return

                Dim TabRect As Rectangle = TabControlMain.GetTabRect(TabControlMain.TabPages.IndexOf(hover_Tab))
                TabRect.Inflate(-3, -3)
                If TabRect.Contains(TabControlMain.PointToClient(New Point(e.X, e.Y))) Then
                    SwapTabPages(drag_tab, hover_Tab)
                    TabControlMain.SelectedTab = drag_tab
                End If
            End If
        End If


    End Sub


#End Region

#Region "Whois Backgroundworker"

    Private Sub bgw_whois_DoWork(sender As Object, e As System.ComponentModel.DoWorkEventArgs) Handles bgw_whois.DoWork

        Dim _whois As New WhoisResolver
        Dim _country_code As String = String.Empty
        Dim _server As String = String.Empty

        _server = CStr(e.Argument)

        Try
            _country_code = _whois.Resolve(_server)

        Catch ex As Exception
            'fehler abfangen
        Finally
            bgw_whois.ReportProgress(0, _country_code)
        End Try

    End Sub

    Private Sub bgw_whois_ProgressChanged(sender As Object, e As System.ComponentModel.ProgressChangedEventArgs) Handles bgw_whois.ProgressChanged

        Dim _country_code As String = String.Empty
        Dim _flag As Image
        Dim _tools As New Tools

        Try

            _country_code = CStr(e.UserState)

            If Not String.IsNullOrEmpty(_country_code) Then

                _flag = _tools.DownloadImage("http://n1.dlcache.com/flags/" & _country_code.ToLower & ".gif")

                If Not _flag Is Nothing Then
                    tsdbb_server_information.DropDownItems.Add("Serverstandort: " & _country_code, _flag)
                End If

            End If


        Catch ex As Exception

        Finally
            tsdbb_server_information.Visible = True
        End Try

    End Sub

#End Region


    Private Sub frm_main_Resize(sender As Object, e As EventArgs) Handles MyBase.Resize

        Try
            If Not _settings Is Nothing Then

                If _settings.Send2Tray = True Then

                    If Me.WindowState = FormWindowState.Minimized Then
                        Me.WindowState = FormWindowState.Minimized
                        NotifyIcon.Visible = True
                        Me.Hide()

                        NotifyIcon.BalloonTipIcon = ToolTipIcon.Info
                        NotifyIcon.BalloonTipText = "SFDL.NET 2 wurde ins Tray minimiert!"
                        NotifyIcon.BalloonTipTitle = "SFDL.NET 2"
                        NotifyIcon.ShowBalloonTip(300)

                    End If

                End If

            End If

        Catch ex As Exception
        End Try


    End Sub

    Private Sub NotifyIcon_MouseClick(sender As Object, e As MouseEventArgs) Handles NotifyIcon.MouseClick

        Dim _string_info As New StringBuilder

        Try

            If e.Clicks < 2 Then


                If _var_download_running = True Then

                    System.Threading.Thread.Sleep(500)

                    If e.Button = Windows.Forms.MouseButtons.Left And e.Clicks = 0 Then

                        _string_info.AppendLine("")
                        _string_info.AppendLine("Geschwindigkeit: " & _var_download_speed)
                        _string_info.AppendLine("Fortschritt: " & _var_download_progress)
                        _string_info.AppendLine("ETA: " & _var_remaining_time)

                        NotifyIcon.BalloonTipIcon = ToolTipIcon.Info
                        NotifyIcon.BalloonTipTitle = "Download Info"
                        NotifyIcon.BalloonTipText = _string_info.ToString
                        NotifyIcon.ShowBalloonTip(3000)

                    End If

                Else
                    NotifyIcon.BalloonTipIcon = ToolTipIcon.Info
                    NotifyIcon.BalloonTipTitle = "Download Info"
                    NotifyIcon.BalloonTipText = "Kein Download aktiv!"
                    NotifyIcon.ShowBalloonTip(3000)

                End If

            End If

        Catch ex As Exception
        End Try

    End Sub

    Private Sub NotifyIcon_MouseDoubleClick(sender As Object, e As MouseEventArgs) Handles NotifyIcon.MouseDoubleClick

        Try

            Me.Show()
            Me.WindowState = FormWindowState.Normal
            NotifyIcon.Visible = False

        Catch ex As Exception
        End Try

    End Sub

    Private Sub MaximierenToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles MaximierenToolStripMenuItem.Click

        Me.Show()
        Me.WindowState = FormWindowState.Normal
        NotifyIcon.Visible = False

    End Sub


#Region "Public Funktionen und Subs"

    Public Function AddTask(ByVal _taskname As String) As Integer

        Dim _rt As Integer = 0

        Try

            If Me.InvokeRequired Then
                Me.Invoke(Sub() Me.AddTask(_taskname))
            Else

                Try

                    Monitor.Enter(_obj_lock_Task)

                    Dim _menu_item As New ToolStripButton

                    StatusStripTasks.Visible = True

                    _menu_item.ToolTipText = _taskname
                    _menu_item.Image = My.Resources.indicator_white

                    StatusStripTasks.Items.Add(_menu_item)

                    _rt = StatusStripTasks.Items.Count - 1


                Catch ex As Exception

                Finally
                    Monitor.Exit(_obj_lock_Task)
                End Try

            End If

        Catch ex As Exception

        End Try

        Return _rt

    End Function

    Public Sub SetTaskComplete(ByVal _taskid As Integer, ByVal _status_text As String, Optional ByVal _error As Boolean = False)

        Try

            If Me.InvokeRequired Then
                Me.Invoke(Sub() Me.SetTaskComplete(_taskid, _status_text, _error))
            Else

                Try

                    Monitor.Enter(_obj_lock_Task)

                    Dim _menu_item As New ToolStripButton

                    StatusStripTasks.Visible = True

                    _menu_item = CType(StatusStripTasks.Items.Item(_taskid), ToolStripButton)

                    _menu_item.ToolTipText = _status_text


                    If _error = True Then
                        _menu_item.Image = My.Resources.fehler
                    Else
                        _menu_item.Image = My.Resources.tick
                    End If

                    _cleanup_timer = New Timer(AddressOf CleanUpTimer, _taskid, 5000, System.Threading.Timeout.Infinite)



                Catch ex As Exception

                Finally
                    Monitor.Exit(_obj_lock_Task)
                End Try

            End If

        Catch ex As Exception

        End Try

    End Sub

#End Region

End Class

