﻿Option Strict On

Namespace FTP
    ''' <summary>
    ''' The itemType of item as reported by the FTP server.
    ''' </summary>
    ''' <remarks>
    ''' Data transmitted from the FTP server after a directory list operation is usually a item itemType of Directory or File.  Unix 
    ''' systems also support additional directory item types such as symbolic links and named sockets.  Not all FTP servers will report
    ''' enough information to determine the file itemType.  In such cases a file itemType of Unknown is specified.
    ''' </remarks>
    Public Enum FtpItemType
        ''' <summary>
        ''' Directory item.
        ''' </summary>
        Directory
        ''' <summary>
        ''' File item.
        ''' </summary>
        File
        ''' <summary>
        ''' Symbolic link item.
        ''' </summary>
        SymbolicLink
        ''' <summary>
        ''' Block special file item.
        ''' </summary>
        BlockSpecialFile
        ''' <summary>
        ''' Character special file item.
        ''' </summary>
        CharacterSpecialFile
        ''' <summary>
        ''' Name socket item.
        ''' </summary>
        NamedSocket
        ''' <summary>
        ''' Domain socket item.
        ''' </summary>
        DomainSocket
        ''' <summary>
        ''' Unknown item.  The system was unable to determine the itemType of item.
        ''' </summary>
        Unknown
    End Enum
End Namespace