﻿Option Strict On

Namespace FTP
    ''' <summary>
    ''' Enumeration representing all the various response codes from a FTP server.
    ''' </summary>
    Public Enum FtpResponseCode As Integer
        ''' <summary>
        ''' No response was received from the server.
        ''' </summary>
        None = 0
        ''' <summary>
        ''' The command was executed sucessfully (200).
        ''' </summary>
        CommandOkay = 200
        ''' <summary>
        ''' A syntax error occurred because the command was not recognized (500).
        ''' </summary>
        SyntaxErrorCommandUnrecognized = 500
        ''' <summary>
        ''' A syntax error occurred because the input parameters or arguments for the command are invalid (501).
        ''' </summary>
        SyntaxErrorInParametersOrArguments = 501
        ''' <summary>
        ''' The command is considered superfluous and not implemented by the FTP server (202).
        ''' </summary>
        CommandNotImplementedSuperfluousAtThisSite = 202
        ''' <summary>
        ''' The command is not implement by the FTP server (502).
        ''' </summary>
        CommandNotImplemented = 502
        ''' <summary>
        ''' A bad sequence of commands was issued (503).
        ''' </summary>
        BadSequenceOfCommands = 503
        ''' <summary>
        ''' The command does not support the supplied parameter (504).
        ''' </summary>
        CommandNotImplementedForThatParameter = 504
        ''' <summary>
        ''' Restart marker reply (110).  MARK yyyy = mmmm  Where yyyy is User-process data 
        ''' stream marker, and mmmm server's equivalent marker (note the spaces between
        ''' markers and "=").
        ''' </summary>
        RestartMarkerReply = 110
        ''' <summary>
        ''' System status or system help reply (211).
        ''' </summary>
        SystemStatusOrHelpReply = 211
        ''' <summary>
        ''' Directory status (212).
        ''' </summary>
        DirectoryStatus = 212
        ''' <summary>
        ''' File status (213).
        ''' </summary>
        FileStatus = 213
        ''' <summary>
        ''' Help message (214).  On how to use the server or the meaning of a particular
        ''' non-standard command.  This reply is useful only to the human user.
        ''' </summary>
        HelpMessage = 214
        ''' <summary>
        ''' Name system type where Name is an official system name from the list in the
        ''' Assigned Numbers document (215).
        ''' </summary>
        NameSystemType = 215
        ''' <summary>
        ''' Service ready in xxx minutes (120).
        ''' </summary>
        ServiceReadyInxxxMinutes = 120
        ''' <summary>
        ''' Service is now ready for new user (220).
        ''' </summary>
        ServiceReadyForNewUser = 220
        ''' <summary>
        ''' Service is closing control connection (221).
        ''' </summary>
        ServiceClosingControlConnection = 221
        ''' <summary>
        ''' Service not available, closing control connection (421). This may be a reply to any 
        ''' command if the service knows it must shut down.
        ''' </summary>
        ServiceNotAvailableClosingControlConnection = 421
        ''' <summary>
        ''' Data connection already open; transfer starting (125).
        ''' </summary>
        DataConnectionAlreadyOpenSoTransferStarting = 125
        ''' <summary>
        ''' Data connection open so no transfer in progress (225).
        ''' </summary>
        DataConnectionOpenSoNoTransferInProgress = 225
        ''' <summary>
        ''' Can not open data connection (425).
        ''' </summary>
        CannotOpenDataConnection = 425
        ''' <summary>
        ''' Requested file action successful (for example, file transfer or file abort) (226).
        ''' </summary>
        ClosingDataConnection = 226
        ''' <summary>
        ''' Connection closed therefore the transfer was aborted (426).
        ''' </summary>
        ConnectionClosedSoTransferAborted = 426
        ''' <summary>
        ''' Entering Passive Mode (h1,h2,h3,h4,p1,p2) (227).
        ''' </summary>
        EnteringPassiveMode = 227
        ''' <summary>
        ''' User logged in, proceed (230).
        ''' </summary>
        UserLoggedIn = 230
        ''' <summary>
        ''' User is not logged in.  Command not accepted (530).
        ''' </summary>
        NotLoggedIn = 530
        ''' <summary>
        ''' The user name was accepted but the password must now be supplied (331).
        ''' </summary>
        UserNameOkayButNeedPassword = 331
        ''' <summary>
        ''' An account is needed for login (332).
        ''' </summary>
        NeedAccountForLogin = 332
        ''' <summary>
        ''' An account is needed for storing file on the server (532).
        ''' </summary>
        NeedAccountForStoringFiles = 532
        ''' <summary>
        ''' File status okay; about to open data connection (150).
        ''' </summary>
        FileStatusOkaySoAboutToOpenDataConnection = 150
        ''' <summary>
        ''' Requested file action okay, completed (250).
        ''' </summary>
        RequestedFileActionOkayAndCompleted = 250
        ''' <summary>
        ''' The pathname was created (257).
        ''' </summary>
        PathNameCreated = 257
        ''' <summary>
        ''' Requested file action pending further information (350).
        ''' </summary>
        RequestedFileActionPendingFurtherInformation = 350
        ''' <summary>
        ''' Requested file action not taken (450).  
        ''' </summary>
        RequestedFileActionNotTaken = 450
        ''' <summary>
        ''' Requested file action not taken (550).  File unavailable (e.g., file busy).
        ''' </summary>
        RequestedActionNotTakenFileUnavailable = 550
        ''' <summary>
        ''' Requested action aborted (451). Local error in processing.
        ''' </summary>
        RequestedActionAbortedDueToLocalErrorInProcessing = 451
        ''' <summary>
        ''' Requested action aborted (551). Page type unknown.
        ''' </summary>
        RequestedActionAbortedPageTypeUnknown = 551
        ''' <summary>
        ''' Requested action not taken (452).  Insufficient storage space in system.
        ''' </summary>
        RequestedActionNotTakenInsufficientStorage = 452
        ''' <summary>
        ''' Requested file action aborted (552).  Exceeded storage allocation (for current directory or dataset).
        ''' </summary>
        RequestedFileActionAbortedExceededStorageAllocation = 552
        ''' <summary>
        ''' Requested action not taken (553).  File name not allowed.
        ''' </summary>
        RequestedActionNotTakenFileNameNotAllowed = 553
        ''' <summary>
        ''' Secure authentication Okay (234).
        ''' </summary>
        AuthenticationCommandOkay = 234
        ''' <summary>
        ''' SSL service is Unavailable (431).
        ''' </summary>
        ServiceIsUnavailable = 431
    End Enum
End Namespace