﻿Option Strict On

Namespace FTP
    ''' <summary>
    ''' The type of data transfer mode (e.g. Active or Passive).
    ''' </summary>
    ''' <remarks>
    ''' The default setting is Passive data transfer mode.  This mode is widely used as a
    ''' firewall friendly setting for the FTP clients operating behind a firewall.
    ''' </remarks>
    Public Enum TransferMode As Integer
        ''' <summary>
        ''' Active transfer mode.  In this mode the FTP server initiates a connection to the client when transfering data.
        ''' </summary>
        ''' <remarks>This transfer mode may not work when the FTP client is behind a firewall and is accessing a remote FTP server.</remarks>
        Active
        ''' <summary>
        ''' Passive transfer mode.  In this mode the FTP client initiates a connection to the server when transfering data.
        ''' </summary>
        ''' <remarks>
        ''' This transfer mode is "firewall friendly" and generally allows an FTP client behind a firewall to access a remote FTP server.
        ''' This mode is recommended for most data transfers.
        ''' </remarks>
        Passive
    End Enum
End Namespace