'
' *  Authors:  Benton Stark
' * 
' *  Copyright (c) 2007-2012 Starksoft, LLC (http://www.starksoft.com) 
' *
' * Permission is hereby granted, free of charge, to any person obtaining a copy
' * of this software and associated documentation files (the "Software"), to deal
' * in the Software without restriction, including without limitation the rights
' * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' * copies of the Software, and to permit persons to whom the Software is
' * furnished to do so, subject to the following conditions:
' * 
' * The above copyright notice and this permission notice shall be included in
' * all copies or substantial portions of the Software.
' * 
' * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' * THE SOFTWARE.
' * 
' 



Namespace FTP
    ''' <summary>
    ''' Event arguments to facilitate the transfer complete event.
    ''' </summary>
    Public Class TransferCompleteEventArgs
        Inherits EventArgs

        Private _bytesTransferred As Long
        Private _bytesPerSecond As Integer
        Private _elapsedTime As TimeSpan

        ''' <summary>
        ''' Constructor.
        ''' </summary>
        ''' <param name="bytesTransferred">The total number of bytes transferred.</param>
        ''' <param name="bytesPerSecond">The data transfer speed in bytes per second.</param>
        ''' <param name="elapsedTime">The time that has elapsed since the data transfer started.</param>
        Public Sub New(bytesTransferred As Long, bytesPerSecond As Integer, elapsedTime As TimeSpan)
            _bytesTransferred = bytesTransferred
            _bytesPerSecond = bytesPerSecond
            _elapsedTime = elapsedTime
        End Sub

        ''' <summary>
        ''' The total number of bytes transferred.
        ''' </summary>
        Public ReadOnly Property BytesTransferred() As Long
            Get
                Return _bytesTransferred
            End Get
        End Property

        ''' <summary>
        ''' Gets the data transfer speed in bytes per second.
        ''' </summary>
        Public ReadOnly Property BytesPerSecond() As Integer
            Get
                Return _bytesPerSecond
            End Get
        End Property

        ''' <summary>
        ''' Gets the data transfer speed in kilobytes per second.
        ''' </summary>
        Public ReadOnly Property KilobytesPerSecond() As Integer
            Get
                Return _bytesPerSecond \ 1024
            End Get
        End Property

        ''' <summary>
        ''' Gets the time that has elapsed since the data transfer started.
        ''' </summary>
        Public ReadOnly Property ElapsedTime() As TimeSpan
            Get
                Return _elapsedTime
            End Get
        End Property


    End Class
End Namespace
