'
' *  Authors:  Benton Stark
' * 
' *  Copyright (c) 2007-2012 Starksoft, LLC (http://www.starksoft.com) 
' *
' * Permission is hereby granted, free of charge, to any person obtaining a copy
' * of this software and associated documentation files (the "Software"), to deal
' * in the Software without restriction, including without limitation the rights
' * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' * copies of the Software, and to permit persons to whom the Software is
' * furnished to do so, subject to the following conditions:
' * 
' * The above copyright notice and this permission notice shall be included in
' * all copies or substantial portions of the Software.
' * 
' * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' * THE SOFTWARE.
' * 
' 


Imports System.Runtime.Serialization

Namespace FTP

    ''' <summary>
    ''' This exception is thrown when a general FTP exception occurs.
    ''' </summary>
    <Serializable> _
    Public Class FtpException
        Inherits Exception
        Private _response As New FtpResponse()

        ''' <summary>
        ''' Constructor.
        ''' </summary>
        Public Sub New()
        End Sub

        ''' <summary>
        ''' Constructor.
        ''' </summary>
        ''' <param name="message">Exception message text.</param>
        Public Sub New(message As String)
            MyBase.New(message)
        End Sub


        ''' <summary>
        ''' Constructor.
        ''' </summary>
        ''' <param name="message">Exception message text.</param>
        ''' <param name="innerException">The inner exception object.</param>
        Public Sub New(message As String, innerException As Exception)
            MyBase.New(message, innerException)
        End Sub

        ''' <summary>
        ''' Constructor.
        ''' </summary>
        ''' <param name="message">Exception message text.</param>
        ''' <param name="response">Response object.</param>
        ''' <param name="innerException">The inner exception object.</param>
        Public Sub New(message As String, response As FtpResponse, innerException As Exception)
            MyBase.New(message, innerException)
            _response = response
        End Sub


        ''' <summary>
        ''' Constructor.
        ''' </summary>
        ''' <param name="message">Exception message text.</param>
        ''' <param name="response">Ftp response object.</param>
        Public Sub New(message As String, response As FtpResponse)
            MyBase.New(message)
            _response = response
        End Sub


        ''' <summary>
        ''' Constructor.
        ''' </summary>
        ''' <param name="info">Serialization information.</param>
        ''' <param name="context">Stream context information.</param>
        Protected Sub New(info As SerializationInfo, context As StreamingContext)
            MyBase.New(info, context)
        End Sub

        ''' <summary>
        ''' Gets the last FTP response if one is available.
        ''' </summary>
        Public ReadOnly Property LastResponse() As FtpResponse
            Get
                Return _response
            End Get
        End Property


        ''' <summary>
        ''' Gets a message that describes the current exception.
        ''' </summary>
        Public Overrides ReadOnly Property Message() As String
            Get
                If _response.Code = FtpResponseCode.None Then
                    Return MyBase.Message
                Else
                    Return [String].Format("{0}  (Last Server Response: {1}  {2})", MyBase.Message, _response.Text, _response.Code)
                End If


            End Get
        End Property
    End Class

End Namespace
