'
' *  Authors:  Benton Stark
' * 
' *  Copyright (c) 2007-2012 Starksoft, LLC (http://www.starksoft.com) 
' *
' * Permission is hereby granted, free of charge, to any person obtaining a copy
' * of this software and associated documentation files (the "Software"), to deal
' * in the Software without restriction, including without limitation the rights
' * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' * copies of the Software, and to permit persons to whom the Software is
' * furnished to do so, subject to the following conditions:
' * 
' * The above copyright notice and this permission notice shall be included in
' * all copies or substantial portions of the Software.
' * 
' * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' * THE SOFTWARE.
' * 
' 



Imports System.Collections.Generic
Imports System.Text

Namespace FTP
    ''' <summary>
    ''' Ftp feature argument item class.
    ''' </summary>
    Public Class FtpFeatureArgument
        Private _name As String
        Private _isDefault As Boolean

        ''' <summary>
        ''' Constructor to create a new ftp feature.
        ''' </summary>
        ''' <param name="name">Feature name.</param>
        Public Sub New(name As String)
            If [String].IsNullOrEmpty(name) Then
                Throw New ArgumentNullException("name", "must have a value")
            End If

            _name = name.Trim()
            Parse()
        End Sub

        ''' <summary>
        ''' Feature name.
        ''' </summary>
        Public ReadOnly Property Name() As String
            Get
                Return _name
            End Get
        End Property

        ''' <summary>
        ''' Default value indicator. 
        ''' </summary>
        ''' <remarks>Some feature arguments may be marked as a default argument by the FTP server.</remarks>
        Public ReadOnly Property IsDefault() As Boolean
            Get
                Return _isDefault
            End Get
        End Property

        Private Sub Parse()
            Dim a As Char() = _name.ToCharArray()
            Dim last As Integer = a.Length - 1
            If a(last) = "*"c Then
                _isDefault = True
                _name = _name.Substring(0, last)
            End If

        End Sub

    End Class
End Namespace
