'
' *  Authors:  Benton Stark
' * 
' *  Copyright (c) 2007-2012 Starksoft, LLC (http://www.starksoft.com) 
' *
' * Permission is hereby granted, free of charge, to any person obtaining a copy
' * of this software and associated documentation files (the "Software"), to deal
' * in the Software without restriction, including without limitation the rights
' * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' * copies of the Software, and to permit persons to whom the Software is
' * furnished to do so, subject to the following conditions:
' * 
' * The above copyright notice and this permission notice shall be included in
' * all copies or substantial portions of the Software.
' * 
' * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' * THE SOFTWARE.
' * 
' 


Imports System.Collections.Generic
Imports System.Text


Namespace FTP
    ''' <summary>
    ''' Thread safe FtpResponse queue object.
    ''' </summary>
    Friend Class FtpResponseQueue
        Private _queue As New Queue(Of FtpResponse)(10)

        ''' <summary>
        ''' Gets the number of elements contained in the FtpResponseQueue.
        ''' </summary>
        Public ReadOnly Property Count() As Integer
            Get
                SyncLock Me
                    Return _queue.Count
                End SyncLock
            End Get
        End Property





        ''' <summary>
        ''' Adds an Response object to the end of the FtpResponseQueue.
        ''' </summary>
        ''' <param name="item">An FtpResponse item.</param>
        Public Sub Enqueue(item As FtpResponse)
            SyncLock Me
                _queue.Enqueue(item)
            End SyncLock
        End Sub

        ''' <summary>
        ''' Removes and return the FtpResponse object at the beginning of the FtpResponseQueue.
        ''' </summary>
        ''' <return>FtpResponse object at the beginning of the FtpResponseQueue</return>
        Public Function Dequeue() As FtpResponse
            SyncLock Me
                Return _queue.Dequeue()
            End SyncLock
        End Function





    End Class
End Namespace
